package modules.bibd;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;

public class MatrixPanel extends JPanel {
  double[][] m;
  JLabel[][] mLabel;
  JLabel[][] rowLabel;
  JLabel[][] colLabel;
  JLabel cornerLabel;
  
  int rows;
  int columns;

  GridLayout matrixLayout;

  boolean brackets;

  private JPanel mPanel;

  public MatrixPanel(double[][] matrix) {
    this(matrix, true, (JLabel[]) null, (JLabel[]) null);
  }

  public MatrixPanel(int m, int n) {
    this(new double[m][n], true, (JLabel[]) null, (JLabel[]) null);
  }

  public MatrixPanel(double[][] matrix,
                     JLabel[] rl, JLabel[] cl) {
    this(matrix,true,rl,cl);
  }

  public MatrixPanel(double[][] matrix, boolean b, 
                     JLabel[] rl, JLabel[] cl) {
    super();
    brackets = b;
    setMatrix(matrix,rl,cl,true);
  }

  public MatrixPanel(double[][] matrix, boolean b, 
                     String[] rl, String[] cl) {
    super();
    brackets = b;    
    setMatrix(matrix,rl,cl,true);
 }

  public void setMatrix(double[][] matrix,
                        String[] rs, String[] cs,
                        boolean display) {
    JLabel[] rl = new JLabel[rs.length];
    JLabel[] cl = new JLabel[cs.length];

    for(int i = 0; i < rs.length; i++) {
      rl[i] = new JLabel(rs[i]);
    }
      
    for(int j = 0; j < cs.length; j++) {
      cl[j] = new JLabel(cs[j]);      
    }

    this.setMatrix(matrix, rl, cl, display);

  }

  public void setMatrix(double[][] matrix,
                        JLabel[] rl, JLabel[] cl,
                        boolean display) {
    if(mPanel != null)
      mPanel.removeAll();

    m = matrix;
    rows = m.length;
    columns = m[0].length;
    
    JPanel nm;

    if(brackets)
      nm = 
        new JPanel(matrixLayout = new GridLayout(rows+2,columns+2,12,10)) {
          public void paintComponent(Graphics g) {
            super.paintComponent(g);
            
            Graphics2D g2d = (Graphics2D) g;
            
            GridLayout l = (GridLayout) getLayout();
            
            int cw = (getWidth()+l.getHgap())/(l.getColumns());
            int rw = (getHeight()+l.getVgap())/(l.getRows());
            
            g2d.setStroke(new BasicStroke(2));
            g2d.setColor(Color.BLACK);
            
            g2d.draw(new Line2D.Float(cw,rw,cw,getHeight()-rw));
            g2d.draw(new Line2D.Float(cw,rw,cw+5,rw));
            g2d.draw(new Line2D.Float(cw,getHeight()-rw,cw+5,getHeight()-rw));
            
            g2d.draw(new Line2D.Float(getWidth()-cw,rw,
                                      getWidth()-cw,getHeight()-rw));
            g2d.draw(new Line2D.Float(getWidth()-cw,rw,getWidth()-cw-5,rw));
            g2d.draw(new Line2D.Float(getWidth()-cw,getHeight()-rw,
                                      getWidth()-cw-5,getHeight()-rw));
          }
        };    
    else
      nm = 
        new JPanel(matrixLayout = new GridLayout(rows+2,columns+2,12,10));
    
    rowLabel = new JLabel[2][rows];
    colLabel = new JLabel[2][columns];

    rowLabel[0] = rl;
    colLabel[0] = cl;
      
    for(int i = 0; i < rows; i++) {
      //rowLabel[0][i] = new JLabel();
      rowLabel[1][i] = new JLabel();
    }
      
    for(int j = 0; j < columns; j++) {
      //colLabel[0][j] = new JLabel();
      colLabel[1][j] = new JLabel();
    }
      
    mLabel = new JLabel[rows][columns];
      
    nm.add(new JLabel());
    for(int j = 0; j < columns; j++)
      nm.add(colLabel[0][j]);
    nm.add(new JLabel());
      
      
    for(int i = 0; i < rows; i++) {
      rowLabel[0][i].setHorizontalAlignment(SwingConstants.CENTER);
      nm.add(rowLabel[0][i]);
      for(int j = 0; j < columns; j++) {
        if((double)((int)m[i][j]) == m[i][j])
          nm.add(mLabel[i][j] = new JLabel((int)m[i][j]+"",
                                               SwingConstants.CENTER));
        else
          nm.add(mLabel[i][j] = new JLabel(m[i][j]+"",
                                               SwingConstants.CENTER));
      }
          
      rowLabel[1][i].setHorizontalAlignment(SwingConstants.CENTER);
      nm.add(rowLabel[1][i]);
    }
      
    nm.add(new JLabel());
    for(int j = 0; j < columns; j++) {
      colLabel[1][j].setHorizontalAlignment(SwingConstants.CENTER);
      nm.add(colLabel[1][j]);
    }

    nm.add(cornerLabel = new JLabel());
    mPanel = nm;
    
    if(display) {
      add(mPanel);
      validate();
      repaint();
    }
  }

  public void setRowLabels(String[] l) {
    for(int i = 0; i < rowLabel[0].length; i++)
      rowLabel[0][i].setText(l[i]);
  }

  public void setColumnLabels(String[] l) {
    for(int i = 0; i < colLabel[0].length; i++)
      colLabel[0][i].setText(l[i]);
  }

  public void setRowLabel(int w, int i, String s) {
    rowLabel[w][i].setText(s);
  }

  public void setRowLabel(int w, int i, int n) {
    setRowLabel(w,i,n+"");
  }

  public void setColumnLabel(int w, int j, String s) {
    colLabel[w][j].setText(s);
  }

  public void setColumnLabel(int w, int j, int n) {
    setColumnLabel(w,j,n+"");
  }

  public void setCornerLabel(String s) {
    cornerLabel.setText(s);
  }

  public void setCornerLabel(int n) {
    setCornerLabel(n+"");
  }

  public double getEntry(int i, int j) {
    return m[i][j];
  }

  public void colorRowLabel(int w, int i, Color c) {
    rowLabel[w][i].setForeground(c);
  }

  public void colorColumnLabel(int w, int j, Color c) {
    colLabel[w][j].setForeground(c);
  }

  public void colorRow(int i, Color c, boolean l) {
    for(int j = 0; j < columns; j++)
      mLabel[i][j].setForeground(c);

    if(l) {
      colorRowLabel(0,i,c);
      colorRowLabel(1,i,c);
    }
  }

  public void colorRow(int i, Color c) {
    colorRow(i,c,false);
  }

  public void colorColumn(int j, Color c, boolean l) {
    for(int i = 0; i < rows; i++)
      mLabel[i][j].setForeground(c);

    if(l) {
      colorColumnLabel(0,j,c);
      colorColumnLabel(1,j,c);
    }
  }

  public void colorColumn(int j, Color c) {
    colorColumn(j,c,false);
  }

  public void colorEntry(int i, int j, Color c) {
    mLabel[i][j].setForeground(c);
  }

  public void removeRow(int i, boolean series) {
    double[][] m2 = new double[rows-1][columns];
    System.arraycopy(m, 0, m2, 0, i);
    System.arraycopy(m, i+1, m2, i, m2.length-i);

    JLabel rl[] = new JLabel[rows-1];
    System.arraycopy(rowLabel[0], 0, rl, 0, i);
    System.arraycopy(rowLabel[0], i+1, rl, i, rl.length-i);
    
    setMatrix(m2,rl,colLabel[0],!series);
  }

  public void removeColumn(int j, boolean series) {
    double[][] m2 = new double[rows][columns-1];
    for(int i = 0; i < rows; i++) {
      m2[i] = new double[columns-1];
      System.arraycopy(m[i], 0, m2[i], 0, j);
      System.arraycopy(m[i], j+1, m2[i], j, m2[i].length-j);
    }

    JLabel cl[] = new JLabel[columns-1];
    System.arraycopy(colLabel[0], 0, cl, 0, j);
    System.arraycopy(colLabel[0], j+1, cl, j, cl.length-j);    
    
    setMatrix(m2,rowLabel[0],cl,!series);
  }

  public int getRows() {
    return rows;
  }

  public int getColumns() {
    return columns;
  }



}