package modules.binomial;

import javax.swing.*;
import java.awt.*;
import java.awt.font.*;

import vcp.utilities.UnicodeUtil;
import vcp.utilities.NoFontFoundException;
import vcp.utilities.ScrollableTextArea;
import vcp.animation.*;
import vcp.mathnotation.*;

import info.clearthought.layout.TableLayout;

public class BinomialVisualization extends JPanel 
  implements 
    vcp.utilities.ScrollableTextAreaConstants {
  
  static final Color JAKE = Color.magenta;
  static final Color KARL = Color.cyan;
  static final Color RYAN = Color.green;

  AnimationCanvas canvas;

  ControlPanel cp;

  //JLabel[] combos;
  GraphicLabel[] combos;

  ScrollableTextArea details;

  JLabel rlabel;

  public BinomialVisualization() {
    super(new TableLayout
          (new double[][]
            {{TableLayout.PREFERRED,
              TableLayout.FILL},
             {TableLayout.PREFERRED,
              TableLayout.FILL,
              TableLayout.PREFERRED}}));

    /*
    double[] c = {TableLayout.PREFERRED};
    double[] r = new double[27];

    for(int i = 0; i < r.length; i++) {
      r[i] = TableLayout.PREFERRED;
    }

    combos = new JLabel[27];
    String[] lang = new String[] {"C","J\u2081","J\u2082"};

    JPanel decisions = new JPanel
      (new TableLayout(c,r));
    */

    MathSymbol[] lang = new MathSymbol[]
      {MathSymbol.getTextSymbol("C",Font.PLAIN),
       MathSymbol.getSubscript
       (MathSymbol.getTextSymbol("J",Font.PLAIN).toMathExpression(),
        MathSymbol.getTextSymbol("1").toMathExpression()),
       MathSymbol.getSubscript
       (MathSymbol.getTextSymbol("J",Font.PLAIN).toMathExpression(),
        MathSymbol.getTextSymbol("2").toMathExpression())};

    MathSymbol comma = MathSymbol.getTextSymbol(",");
    
    combos = new GraphicLabel[27];

    int ci = 0;
    for(int i = 0; i < 3; i++)
      for(int j = 0; j < 3; j++)
        for(int k = 0; k < 3; k++) {
          combos[ci++] = new GraphicLabel
            (new MathExpression
             (new MathSymbol[]
               {lang[i],
                //MathSpace.getNoSpace(),
                //comma,
                lang[j],
                //MathSpace.getNoSpace(),
                //comma,
                lang[k]},14f));
        }
    

    canvas = new AnimationCanvas(600,540,getBackground());
    cp = new SliderControlPanel();

    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));
    details.setMargins(6);

    MathExpression expr = new MathExpression
      (new MathSymbol[]
        {MathSymbol.getSuperscript
         (MathSymbol.getParens
          (MathSymbol.getTextSymbol("x + y")).toMathExpression(),
          MathSymbol.getTextSymbol("n").toMathExpression()),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getSummationSymbol
         ("r=0", "n", new MathExpression
          (new MathSymbol[]
            {MathSymbol.getBinomialCoefficient("n","r"),
             MathSpace.getNoSpace(),
             MathSymbol.getSuperscript("x","r"),
             MathSpace.getNoSpace(),
             MathSymbol.getSuperscript("y","n\u2212r")},1f),
          "sum pf C(n,r)x^(n-r)y^r with r = 0 to n")
        },20f);

    JLabel theorem = new JLabel
      ("The Binomial Theorem:",
       new ImageIcon(expr.toImage(10)),
       JLabel.CENTER);
    theorem.setVerticalTextPosition(JLabel.TOP);
    theorem.setHorizontalTextPosition(JLabel.CENTER);
    rlabel = new JLabel("");


    JPanel people = new JPanel();
    people.setLayout(new BoxLayout(people,BoxLayout.Y_AXIS));
    
    JLabel jakeL = new JLabel("Jake");
    jakeL.setForeground(JAKE.darker());
    JLabel karlL = new JLabel("Karl");
    karlL.setForeground(KARL.darker());
    JLabel ryanL = new JLabel("Ryan");
    ryanL.setForeground(RYAN.darker());

    people.add(jakeL);
    people.add(karlL);
    people.add(ryanL);
    

    add(canvas,"1,0,c,c");
    add(theorem,"0,0,l,b");
    add(people,"0,0,l,c");
    add(rlabel,"0,0,l,t");
    add(details.getJScrollPane(),"0,1,1,1");
    add(cp,"0,2,1,2");
  }

  public void addNotify() {
    super.addNotify();
    
    new BinomialAnimation();
  }

  private class BinomialAnimation extends FPSAnimation {
    DecisionTree lhs;

    DecisionTree[] rhs;

    

    DecisionTree.DecisionNode[] lhsLeaves;
    DecisionTree.DecisionNode[][] rhsLeaves;
    
    public BinomialAnimation() {
      super(cp, 15);
      createTrees();

      Graphics2D g = canvas.getImageGraphics();
      lhs.draw(g,10,270);
      g.dispose();
      
      setPartArray(new AnimationPart[]
        {
          new Part0(),
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5(),
          new Part6(),
          new Part7(),
          new Part8(),
          new Part9(),
          new Part10(),
          new Part11(),
        });
    }

    public void sleepSlow() {
      super.sleep(10);
    }

    private void createTrees() {
      final MathExpression cl = MathSymbol.getTextSymbol
        ("C",Font.PLAIN).toMathExpression(20f);
      final MathExpression j1l = MathSymbol.getSubscript
        (MathSymbol.getTextSymbol("J",Font.PLAIN).toMathExpression(),
         MathSymbol.getTextSymbol("1").toMathExpression())
        .toMathExpression(20f);
      final MathExpression j2l = MathSymbol.getSubscript
        (MathSymbol.getTextSymbol("J",Font.PLAIN).toMathExpression(),
         MathSymbol.getTextSymbol("2").toMathExpression())
        .toMathExpression(20f);

      lhs = new DecisionTree(20);
      
      lhs.root.color = JAKE;
      
      DecisionTree.DecisionNode[] l2;
      
      lhsLeaves = new DecisionTree.DecisionNode[27];
      int li = 0;

      Color[] edges = new Color[] {Color.red, Color.blue, Color.blue};

      addChildren(lhs.root,3,KARL, edges);
      lhs.root.setEdgeLabel(0,cl);
      lhs.root.setEdgeLabel(1,j1l);
      lhs.root.setEdgeLabel(2,j2l);


      l2 = lhs.root.children;
      
      for(int i = 0; i < l2.length; i++) {
        addChildren(l2[i],3,RYAN, edges);
        for(int j = 0; j < l2[i].children.length; j++) {
         addChildren(l2[i].children[j],3,Color.black, edges);
         for(int k = 0; k < l2[i].children[j].children.length; k++)
           lhsLeaves[li++] = l2[i].children[j].children[k];
        }
      }
      
      rhs = new DecisionTree[8];
      rhsLeaves = new DecisionTree.DecisionNode[8][];

      for(int i = 0; i < rhs.length; i++) {
        rhs[i] = new DecisionTree(20);
        rhs[i].root.color = JAKE;
      }

      rhs[0].spacing = 60;
      rhsLeaves[0] = new DecisionTree.DecisionNode[8];
      li = 0;

      edges = new Color[] {Color.blue, Color.blue};

      addChildren(rhs[0].root,2,KARL, edges);
      rhs[0].root.setEdgeLabel(0,j1l);
      rhs[0].root.setEdgeLabel(1,j2l);

      l2 = rhs[0].root.children;
      
      for(int i = 0; i < l2.length; i++) {
        addChildren(l2[i],2,RYAN, edges);
        for(int j = 0; j < l2[i].children.length; j++) {
         addChildren(l2[i].children[j],2,Color.black, edges);
         for(int k = 0; k < l2[i].children[j].children.length; k++)
           rhsLeaves[0][li++] = l2[i].children[j].children[k];
        }
      }

      /*
      for(int i = 0; i < rhsLeaves[0].length; i++) {
        System.out.println("leaf " + i + ": " + rhsLeaves[0][i]);
        System.out.println("parent " + i + ": " + rhsLeaves[0][i].parent);
      }
      */

      rhs[1].spacing = 40;

      addChildren(rhs[1].root, 1, KARL, new Color[] {Color.red});

      addChildren(rhs[1].root.children[0],2,RYAN, edges);
      addChildren(rhs[1].root.children[0].children[0],2,Color.black, edges);
      addChildren(rhs[1].root.children[0].children[1],2,Color.black, edges);

      rhs[1].root.setEdgeLabel(0,cl);
      rhs[1].root.children[0].setEdgeLabel(0,j1l);
      rhs[1].root.children[0].setEdgeLabel(1,j2l);
      rhsLeaves[1] = rhs[1].getLeaves(4);

      
      rhs[2].spacing = 40;

      addChildren(rhs[2].root, 2, KARL, edges);

      addChildren(rhs[2].root.children[0],1,RYAN, 
                  new Color[] {Color.red});
      addChildren(rhs[2].root.children[1],1,RYAN, 
                  new Color[] {Color.red});
 
      addChildren(rhs[2].root.children[0].children[0],2,RYAN, 
                  edges);
      addChildren(rhs[2].root.children[1].children[0],2,RYAN, 
                  edges);

      rhs[2].root.setEdgeLabel(0,j1l);
      rhs[2].root.setEdgeLabel(1,j2l);
      rhs[2].root.children[0].setEdgeLabel(0,cl);
      rhs[2].root.children[1].setEdgeLabel(0,cl);
      rhsLeaves[2] = rhs[2].getLeaves(4);

      rhs[3].spacing = 40;
      rhsLeaves[3] = new DecisionTree.DecisionNode[4];
      li = 0;

      edges = new Color[] {Color.blue, Color.blue};

      addChildren(rhs[3].root,2,KARL, edges);
      
      l2 = rhs[3].root.children;
      
      for(int i = 0; i < l2.length; i++) {
        addChildren(l2[i],2,RYAN, edges);
        for(int j = 0; j < l2[i].children.length; j++) {
         addChildren(l2[i].children[j],1,Color.black, new Color[] {Color.red});
         l2[i].children[j].setEdgeLabel(0,cl);
         
         for(int k = 0; k < l2[i].children[j].children.length; k++)
           rhsLeaves[3][li++] = l2[i].children[j].children[k];
        }
      }
      
      rhs[3].root.setEdgeLabel(0,j1l);
      rhs[3].root.setEdgeLabel(1,j2l);
      //rhs[3].root.children[1].setEdgeLabel(0,cl);


      rhs[4].spacing = 60;

      addChildren(rhs[4].root, 1, KARL, new Color[] {Color.red});
      addChildren(rhs[4].root.children[0], 1, RYAN, 
                  new Color[] {Color.red});
      addChildren(rhs[4].root.children[0].children[0], 2, KARL, 
                  edges);

      rhs[4].root.setEdgeLabel(0,cl);
      rhs[4].root.children[0].children[0].setEdgeLabel(0,j1l);
      rhs[4].root.children[0].children[0].setEdgeLabel(1,j2l);
      rhsLeaves[4] = rhs[4].getLeaves(2);

      rhs[5].spacing = 60;

      addChildren(rhs[5].root, 1, KARL, new Color[] {Color.red});
      addChildren(rhs[5].root.children[0], 2, RYAN, 
                  edges);
      addChildren(rhs[5].root.children[0].children[0], 1, KARL, 
                  new Color[] {Color.red});
      addChildren(rhs[5].root.children[0].children[1], 1, KARL, 
                  new Color[] {Color.red});

      rhs[5].root.setEdgeLabel(0,cl);
      rhs[5].root.children[0].setEdgeLabel(0,j1l);
      rhs[5].root.children[0].setEdgeLabel(1,j2l);     
      rhsLeaves[5] = rhs[5].getLeaves(2);

      rhs[6].spacing = 60;

      addChildren(rhs[6].root, 2, KARL, edges);
      addChildren(rhs[6].root.children[0], 1, RYAN, 
                  new Color[] {Color.red});
      addChildren(rhs[6].root.children[1], 1, RYAN, 
                  new Color[] {Color.red});
      addChildren(rhs[6].root.children[0].children[0], 1, 
                  KARL, 
                  new Color[] {Color.red});
      addChildren(rhs[6].root.children[1].children[0], 1, 
                  KARL, 
                  new Color[] {Color.red});

      rhs[6].root.setEdgeLabel(0,j1l);
      rhs[6].root.setEdgeLabel(1,j2l);
      rhs[6].root.children[0].setEdgeLabel(0,cl);
      rhs[6].root.children[1].setEdgeLabel(0,cl);
      rhsLeaves[6] = rhs[6].getLeaves(2);
      
      rhs[7].spacing = 60;

      addChildren(rhs[7].root, 1, KARL,
                  new Color[] {Color.red});
      addChildren(rhs[7].root.children[0], 1, RYAN,
                  new Color[] {Color.red});
      addChildren(rhs[7].root.children[0].children[0], 1, RYAN,
                  new Color[] {Color.red});

      rhs[7].root.setEdgeLabel(0,cl);      
      rhsLeaves[7] = rhs[7].getLeaves(1);
    }
    
    private void addChildren(DecisionTree.DecisionNode node, 
                             int n, Color c, Color[] e) {
      node.children = new DecisionTree.DecisionNode[n];
      for(int i = 0; i < node.children.length; i++) {
        node.children[i] = new DecisionTree.DecisionNode(node);
        node.children[i].color = c;
        node.setEdge(i,e[i]);
      }
      //node.setEdge(0,Color.red);
      //node.setEdge(1,Color.blue);
      //node.setEdge(2,Color.blue);
      
    }    

    private void drawTree(Graphics2D g, DecisionTree tree, float x, float y) {
      tree.draw(g,x,y);
    }

    private void drawTree(DecisionTree tree, float x, float y) {
      Graphics2D g = canvas.getImageGraphics();
      for(int j = 0; j < combos.length; j++)
        combos[j].draw(g);

      drawTree(g,tree,x,y);
      g.dispose();
      canvas.paintCanvas();
    }

    private void animateRhs(int i, int[] combi) {
      animateRhs(i,combi,10,270, new int[0], new float[0][]);
    }

    private void animateRhs(int i, int[] combi, float x, float y, 
                            int[] oi, float[][]oc) {

      int j;
      for(j = 0; j < rhsLeaves[i].length; j++) {
        if(j > 0)
          rhsLeaves[i][j-1].highlightPath(false);
        //drawTree(rhs[i],10,270);
        //sleepSlow();
        rhsLeaves[i][j].highlightPath(true);
        /*
        for(float alpha = .3f; alpha <= 1; alpha += .1) {
          combos[combi[j]].setAlpha(alpha);
          drawTree(rhs[i],10,270);
          sleep();
        }
        */
        combos[combi[j]].setAlpha(1);
        Graphics2D g = canvas.getImageGraphics(true);
        drawTree(g,rhs[i],x,y);
        for(int k = 0; k < combos.length; k++)
          combos[k].draw(g);

        for(int k = 0; k < oi.length; k++)
          drawTree(g,rhs[oi[k]],oc[k][0],oc[k][1]);
        g.dispose();
        canvas.paintCanvas();


        sleepSlow();
      }

      rhsLeaves[i][j-1].highlightPath(false);
      Graphics2D g = canvas.getImageGraphics(true);
      drawTree(g,rhs[i],x,y);
      for(int k = 0; k < combos.length; k++)
        combos[k].draw(g);
      
      for(int k = 0; k < oi.length; k++)
        drawTree(g,rhs[oi[k]],oc[k][0],oc[k][1]);
      g.dispose();
      canvas.paintCanvas();
      sleepSlow();
    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.setContent("Suppose we have a class of three (n = 3) computer science students. These students: Jake, Karl, and Ryan must choose to work on either one (x = 1) project in C++ or one of two (y = 2) projects in Java. These projects are C, J\u2081, and J\u2082, respectively.\n\nThe visualization uses decision trees. Decision trees branch at nodes where a choice is being made. The branches of a decision tree represent the available choices. In this example, the nodes represent the students making the decisions and the branches represent the possible projects. Each color node corresponds to a single student's decision. Make sure you understand that each student is making only one decision.\n\nWe start by visualizing the left side of the identity for this example which is the number of ways the three students could choose to do their projects,");
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSuperscript
              (MathSymbol.getParens
               (MathSymbol.getTextSymbol("1 + 2")).toMathExpression(),
               MathSymbol.getTextSymbol("3").toMathExpression()),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("27")
             },20f));

        details.append("We will highlight the path for each of the possible ways the three could decide and list these along the right side of the visualization.");
      }
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        Graphics2D g;

        for(int i = 0; i < lhsLeaves.length; i++) {
          if(i > 0)
            lhsLeaves[i-1].highlightPath(false);
          lhsLeaves[i].highlightPath(true);

          g = canvas.getImageGraphics();          

          for(int j = 0; j <= i; j++)
            combos[j].draw(g,lhsLeaves[j].l.x+4,lhsLeaves[j].l.y);
          
          lhs.draw(g,10,270);
          g.dispose();

          canvas.paintCanvas();
          sleepSlow();
        }

        lhsLeaves[lhsLeaves.length - 1].highlightPath(false);
        g = canvas.getImageGraphics();          
        for(int j = 0; j < lhsLeaves.length; j++)
          combos[j].draw(g,lhsLeaves[j].l.x+4,lhsLeaves[j].l.y);
        lhs.draw(g,10,270);
        g.dispose();
        canvas.paintCanvas();

        for(float x = lhsLeaves[0].l.x+4; x <= 550; x += 5) {
          g = canvas.getImageGraphics();
          for(int j = 0; j < combos.length; j++) 
            combos[j].draw(g,x,lhsLeaves[j].l.y);
          lhs.draw(g,10,270);
          g.dispose();
          canvas.paintCanvas();
          sleep();
        }

        for(float alpha = 1; alpha >= .3; alpha -= .08) {
          g = canvas.getImageGraphics();
          for(int j = 0; j < combos.length; j++) {
            combos[j].setAlpha(alpha);
            combos[j].draw(g);
          }
          lhs.draw(g,10,270);
          g.dispose();
          canvas.paintCanvas();
          sleep();          
        }
      }
    }

    private class Part2 implements AnimationPart {
      public void animate() {
        details.append("Now, we will visualize the right side of the identity. Let r be the number of students who will code in C++. We will look at the value of",NEWLINES|SET_MARK);
        details.append
          (MathSymbol.getSummationSymbol
         ("r = 0", "3", new MathExpression
          (new MathSymbol[]
            {MathSymbol.getBinomialCoefficient("3","r"),
             MathSpace.getNoSpace(),
             MathSymbol.getSuperscript("1","r"),
             MathSpace.getNoSpace(),
             MathSymbol.getSuperscript("2","3\u2212r")},1f),
          "sum of C(3,r)1^(3-r)2^r with r = 0 to 3").toMathExpression(20f)
           );

        details.append("by examining the trees produced with each value of r. The current value of r will appear in the upper-left corner of the visualization. The paths will be highlighted again, and the corresponding decisions will be highlighted in the list we created from the first tree.",SCROLL);
      }
    }
    
    private class Part3 implements AnimationPart {
      public void animate() {
        drawTree(rhs[0],10,270);
        rlabel.setIcon
          (new ImageIcon
           (MathSymbol.getTextSymbol
            ("r = 0").toMathExpression(20f).toImage(10)));
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("3","0"),
              MathSpace.getNoSpace(),
              MathSymbol.getSuperscript("1","0"),
              MathSpace.getNoSpace(),
              MathSymbol.getSuperscript("2","3"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("8")},20f),SET_MARK|SCROLL);        
      }
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        animateRhs(0, new int[]
          {13,14,16,17,22,23,25,26});
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        Graphics2D g = canvas.getImageGraphics();
        for(int j = 0; j < combos.length; j++)
          combos[j].draw(g);
        
        drawTree(g,rhs[1],10,100);
        drawTree(g,rhs[2],10,260);
        drawTree(g,rhs[3],10,420);
        g.dispose();
        canvas.paintCanvas();

        rlabel.setIcon
          (new ImageIcon
           (MathSymbol.getTextSymbol
            ("r = 1").toMathExpression(20f).toImage(10)));

       details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("3","1"),
              MathSpace.getNoSpace(),
              MathSymbol.getSuperscript("1","1"),
              MathSpace.getNoSpace(),
              MathSymbol.getSuperscript("2","2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("12")},20f),SET_MARK|SCROLL);
      }
    }
    private class Part6 implements AnimationPart {
      public void animate() {
         animateRhs(1, new int[]
          {4,5,7,8}, 10, 100, 
                   new int[] {2,3}, new float[][] {{10,260},{10,420}});


        animateRhs(2, new int[]
          {10,11,19,20},10, 260, 
                   new int[] {1,3}, new float[][] {{10,100},{10,420}});


        animateRhs(3, new int[]
          {12,15,21,24}, 10, 420, 
                   new int[] {1,2}, new float[][] {{10,100},{10,260}});
      }
    }

    private class Part7 implements AnimationPart {
      public void animate() {
        Graphics2D g = canvas.getImageGraphics();
        for(int j = 0; j < combos.length; j++)
          combos[j].draw(g);
        
        drawTree(g,rhs[4],10,100);
        drawTree(g,rhs[5],10,260);
        drawTree(g,rhs[6],10,420);
        g.dispose();
        canvas.paintCanvas();
        

        rlabel.setIcon
          (new ImageIcon
           (MathSymbol.getTextSymbol
            ("r = 2").toMathExpression(20f).toImage(10)));

        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("3","2"),
              MathSpace.getNoSpace(),
              MathSymbol.getSuperscript("1","2"),
              MathSpace.getNoSpace(),
              MathSymbol.getSuperscript("2","1"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("6")},20f),SET_MARK|SCROLL);
      }
    }
    private class Part8 implements AnimationPart {
      public void animate() {
        animateRhs(4, new int[]
          {1,2}, 10, 100, 
                   new int[] {5,6}, new float[][] {{10,260},{10,420}});
        
        
        animateRhs(5, new int[]
          {3,6}, 10, 260, 
                   new int[] {4,6}, new float[][] {{10,100},{10,420}});
        
        
        animateRhs(6, new int[]
          {9,18}, 10, 420, 
                   new int[] {4,5}, new float[][] {{10,100},{10,260}});
      }
    }

    private class Part9 implements AnimationPart {
      public void animate() {
        drawTree(rhs[7],10,270);

        rlabel.setIcon
          (new ImageIcon
           (MathSymbol.getTextSymbol
            ("r = 3").toMathExpression(20f).toImage(10)));
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("3","3"),
              MathSpace.getNoSpace(),
              MathSymbol.getSuperscript("1","3"),
              MathSpace.getNoSpace(),
              MathSymbol.getSuperscript("2","0"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("1")},20f),SET_MARK|SCROLL);
      }
    }

    private class Part10 implements AnimationPart {
      public void animate() {
        animateRhs(7, new int[]
          {0});

        details.append
          (MathSymbol.getTextSymbol
           ("8 + 12 + 6 + 1 = 27").toMathExpression(20f),SET_MARK|SCROLL);
      }
    }

    private class Part11 implements AnimationPart {
      public void animate() {
        Graphics2D g = canvas.getImageGraphics();
        for(int k = 0; k < combos.length; k++)
          combos[k].draw(g);
        g.dispose();
        canvas.paintCanvas();

        details.append("It is clear that in this example, ",SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSuperscript
              (MathSymbol.getParens
               (MathSymbol.getTextSymbol("1 + 2")).toMathExpression(),
               MathSymbol.getTextSymbol("3").toMathExpression()),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("27"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSummationSymbol
              ("r = 0", "3", new MathExpression
               (new MathSymbol[]
                 {MathSymbol.getBinomialCoefficient("3","r"),
                  MathSpace.getNoSpace(),
                  MathSymbol.getSuperscript("1","r"),
                  MathSpace.getNoSpace(),
                  MathSymbol.getSuperscript("2","3\u2212r")},1f),
               "sum of C(3,r)1^(3-r)2^r with r = 0 to 3")
             },20f),SCROLL);
      }
    }
    
  }
}
