package modules.fibonacci1;

import javax.swing.*;
import java.awt.*;

import modules.fibonacci.Board;

import vcp.animation.*;
import vcp.mathnotation.*;
import vcp.utilities.ScrollableTextArea;

import info.clearthought.layout.TableLayout;

public class Fibonacci1 extends JPanel 
  implements
    vcp.utilities.ScrollableTextAreaConstants {
  
  AnimationCanvas canvas;

  JPanel animationPanel;

  ScrollableTextArea details;

  Totals totals;

  Fib1Animation ani;

  ControlPanel cp;

  public Fibonacci1() {
    super();

    setLayout(new TableLayout
              (new double[][]
                {{TableLayout.FILL},
                 {TableLayout.PREFERRED,
                  TableLayout.FILL,
                  TableLayout.PREFERRED}}));

    cp = new SliderControlPanel();

    canvas = new AnimationCanvas(210,365,getBackground()); 

    TableLayout tl;

    animationPanel = new JPanel
      (tl = new TableLayout
       (new double[][]
         {{TableLayout.PREFERRED,
           TableLayout.PREFERRED},
          {TableLayout.PREFERRED,
           TableLayout.PREFERRED}}));

    //tl.setHGap(5);

    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(210,365));
    details.setMargins(6);

    totals = new Totals();

    animationPanel.add(canvas,"1,0,c,c");
    animationPanel.add(details.getJScrollPane(),"0,0,c,c");
    animationPanel.add(totals,"0,1,1,1");

    
    MathExpression expr = new MathExpression
      (new MathSymbol[]
        {MathSymbol.getSummationSymbol
         (MathSymbol.getTextSymbol("i = 0").toMathExpression(),
          MathSymbol.getFloorSymbol
          (MathSymbol.getFractionSymbol
           ("n","2").toMathExpression()).toMathExpression(),
          MathSymbol.getBinomialCoefficient("n \u2212 i","i").
          toMathExpression(),
          "sum"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getSubscript("f","n")},30f);
    
    add(new JLabel(new ImageIcon(expr.toImage(10))),"0,0");
    add(new JScrollPane(animationPanel),"0,1,c,c");
    add(cp,"0,2");
  }

  public void addNotify() {
    super.addNotify();

    ani = new Fib1Animation();
  }

  private class Totals extends JPanel {
    public MathExpression expr;

    public Totals() {
      super();

      setPreferredSize(new java.awt.Dimension
                       (500,
                        100));
    }

    public void paintComponent(Graphics g) {
      super.paintComponent(g);

      if(expr != null) {
        expr.draw(g,
                  (float)(getWidth()/2.0 - expr.getWidth(g)/2.0),
                  (float)(getHeight()/2.0 + expr.getHeight(g)/2.0));
      }
    }
  }

  private class Fib1Animation extends Animation {
    static final float BOARD_SIZE = 40;

    Board[] boards;

    MathSymbol[] sym;

    public Fib1Animation() {
      super(cp);

      setPartArray(createPartArray());

      boards = new Board[]
        {new Board(5, BOARD_SIZE),
         new Board(5, BOARD_SIZE),
         new Board(5, BOARD_SIZE),
         new Board(5, BOARD_SIZE),
         new Board(5, BOARD_SIZE),
         new Board(5, BOARD_SIZE),
         new Board(5, BOARD_SIZE),
         new Board(5, BOARD_SIZE),};
      
      boards[1].setDomino(0);
      boards[2].setDomino(1);
      boards[3].setDomino(2);
      boards[4].setDomino(3);
      
      boards[5].setDominoes(new int [] {0,2});
      boards[6].setDominoes(new int [] {0,3});
      boards[7].setDominoes(new int [] {1,3});
      
      float y = 5;
      for(int i = 0; i < boards.length; i++) {
        boards[i].x = 5;
        boards[i].y = y;
        y += BOARD_SIZE + 5;
      }
     
      drawGraphics();
      canvas.paintCanvas();
    }

    protected AnimationPart[] createPartArray() {
      return new AnimationPart[]
        {
          new Part0(),
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5()
        };
    }

    protected void drawGraphics() {
      Graphics2D g = canvas.getImageGraphics();
      drawGraphics(g);
      g.dispose();
    }

    protected void drawGraphics(Graphics2D g) {
      for(int i = 0; i < boards.length; i++) {
        boards[i].draw(g);
      }
    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.setContent("Consider a board of length n = 5. We start with all the possible tilings for this board. We know the number of tilings is equal to the right-hand side of the identity. In the example,");
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSubscript("f","5"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("8")},20f));
      }
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        details.append("Now we will count the left-hand side of the identity.",NEWLINES|SET_MARK|SCROLL);
      }
    }

    private class Part2 implements AnimationPart {
      public void animate() {
        details.append("There is",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("5\u22120","0"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("1")
             },20f));
        details.append("way to tile a 5-board with 0 dominoes.");
        details.scrollToMark();

        sym = new MathSymbol[11];

        sym[0] = MathSymbol.getBinomialCoefficient("5\u22120","0");
        sym[1] = MathSymbol.getTextSymbol("+");

        totals.expr = new MathExpression
          (sym,30f);
        totals.repaint();

        boards[0].highlight(true);
        drawGraphics();
        canvas.paintCanvas();
      }
    }

    private class Part3 implements AnimationPart {
      public void animate() {
        boards[0].highlight(false);
        drawGraphics();
        canvas.paintCanvas();
        
        details.append("There are", NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("5\u22121","1"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("4")
             },20f));
        details.append("ways to tile a 5-board with 1 domino.");
        details.scrollToMark();

        sym[2] = MathSymbol.getBinomialCoefficient("5\u22121","1");
        sym[3] = MathSymbol.getTextSymbol("+");
        totals.expr.setSize(totals.expr.getSize());
        totals.expr.setColor(Color.black);
        totals.repaint();

        for(int i = 1; i < 5; i++)
          boards[i].highlight(true);

        drawGraphics();
        canvas.paintCanvas();

      }
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        for(int i = 1; i < 5; i++)
          boards[i].highlight(false);
        drawGraphics();
        canvas.paintCanvas();

        details.append("There are",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("5\u22122","2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("3")
             },20f));
        details.append("ways to tile a 5-board with 2 dominoes");
        details.scrollToMark();

        sym[4] = MathSymbol.getBinomialCoefficient("5\u22122","2");
        sym[5] = MathSymbol.getTextSymbol("=");
        totals.expr.setSize(totals.expr.getSize());
        totals.expr.setColor(Color.black);
        totals.repaint();

        for(int i = 5; i < 8; i++)
          boards[i].highlight(true);

        drawGraphics();
        canvas.paintCanvas();
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        for(int i = 5; i < 8; i++)
          boards[i].highlight(false);

        drawGraphics();
        canvas.paintCanvas();

        details.append("So, it is clear that the theorem holds true in this example.",NEWLINES|SET_MARK|SCROLL);

        sym[6] = MathSymbol.getSummationSymbol
          (MathSymbol.getTextSymbol("i = 0").toMathExpression(),
           MathSymbol.getFloorSymbol
           (MathSymbol.getFractionSymbol
            ("5","2").toMathExpression()).toMathExpression(),
           MathSymbol.getBinomialCoefficient
           ("5 \u2212 i","i").toMathExpression(),
           "sum");
        sym[7] = MathSymbol.getTextSymbol("=");
        sym[8] = MathSymbol.getTextSymbol("8");
        sym[9] = MathSymbol.getTextSymbol("=");
        sym[10] = MathSymbol.getSubscript("f","5");
        totals.expr.setSize(totals.expr.getSize());
        totals.expr.setColor(Color.black);
        totals.repaint();
      }
    }
  }

}
