package modules.fibonacci3;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt. image.BufferedImage;

import modules.fibonacci.Board;

import vcp.animation.*;
import vcp.mathnotation.*;
import vcp.utilities.ScrollableTextArea;

import info.clearthought.layout.TableLayout;

public class Fibonacci3 extends JPanel 
  implements
    vcp.utilities.ScrollableTextAreaConstants {
  
  ControlPanel cp;

  AnimationCanvas canvas;

  ScrollableTextArea details;

  public Fibonacci3() {
    super(new TableLayout
          (new double[][]
            {{TableLayout.FILL},
             {TableLayout.PREFERRED,
              TableLayout.PREFERRED,
              TableLayout.FILL,
              TableLayout.PREFERRED}}));

    cp = new SliderControlPanel();

    canvas = new AnimationCanvas(335,395,getBackground());
    
    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));
    details.setMargins(6);

    MathExpression expr = new MathExpression
      (new MathSymbol[]
        {MathSymbol.getSubscript("f","m+n"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getSubscript("f","m"),
         MathSpace.getNoSpace(),
         MathSymbol.getSubscript("f","n"),
         MathSymbol.getTextSymbol("+"),
         MathSymbol.getSubscript("f","m-1"),
         MathSpace.getNoSpace(),
         MathSymbol.getSubscript("f","n-1"),},
       30f);

    JLabel theorem = new JLabel
      (new ImageIcon(expr.toImage(10)));

    add(theorem,"0,0,c,c");
    add(new JScrollPane(canvas),"0,1,c,c");
    add(details.getJScrollPane(),"0,2");
    add(cp,"0,3");
  }

  public void addNotify() {
    super.addNotify();

    new Fib3Animation();
  }

  private class Fib3Animation extends FPSAnimation {
    static final float BOARD_SIZE = 25;
    static final int WAIT = 30;
    static final float LEFT_SIDE = 5;
    static final float RIGHT_SIDE = 180;
    static final float BASE = 5 + (BOARD_SIZE+5)*12;
    
    Board[] boards;

    public Fib3Animation() {
      super(cp,60);

      setPartArray(createPartArray());

      boards = new Board[13];

      float y = 5;
      for(int i = 0; i < boards.length; i++) {
        boards[i] = new Board(6,BOARD_SIZE);
        boards[i].x = LEFT_SIDE;
        boards[i].y = y;
        y += BOARD_SIZE + 5;
      }

      /*
      boards[0].breakHighlight(3,true);
      boards[1].breakHighlight(3,true);
      boards[2].breakHighlight(3,true);
      boards[3].breakHighlight(3,true);
      boards[5].breakHighlight(3,true);
      boards[6].breakHighlight(3,true);
      boards[8].breakHighlight(3,true);
      boards[10].breakHighlight(3,true);
      boards[11].breakHighlight(3,true);
      boards[12].breakHighlight(3,true);
      */

      for(int i = 1; i < 6; i++)
        boards[i].setDomino(i-1);

      boards[6].setDominoes(new int[] {0,2});
      boards[7].setDominoes(new int[] {0,3});
      boards[8].setDominoes(new int[] {0,4});

      boards[9].setDominoes(new int[] {1,3});
      boards[10].setDominoes(new int[] {1,4});

      boards[11].setDominoes(new int[] {2,4});
      boards[12].setDominoes(new int[] {0,2,4});


      drawBoards();
    }

    public AnimationPart[] createPartArray() {
      return new AnimationPart[]
        {
          new Part0(),
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5(),
          new Part6(),
        };
    }

    private boolean moveBoards() {
      boolean done = true;
      Board leftTemp[] = new Board[boards.length];
      Board left[] = new Board[boards.length];
      Board rightTemp[] = new Board[boards.length];
      Board right[] = new Board[boards.length];
      int ri = 0;
      int li = 0;

      for(int i = 0; i < boards.length; i++) {
        if(boards[i] != null)
          if(boards[i].x == LEFT_SIDE) {
            leftTemp[(int)(boards[i].y/(BOARD_SIZE+5))] = 
              boards[i];
          }
          else if(boards[i].x == RIGHT_SIDE) {
            rightTemp[(int)(boards[i].y/(BOARD_SIZE+5))] = 
              boards[i];
          }
      }


      for(int i = 0; i < boards.length; i++) {
        if(leftTemp[boards.length - 1 - i] != null)
          left[li++] = leftTemp[boards.length - 1 - i];
        if(rightTemp[i] != null) 
          right[ri++] = rightTemp[i];
      }

      for(int i = 0; i < right.length; i++) {
        if(right[i] != null) {
          if(right[i].y  > (5 + (BOARD_SIZE+5)*i)) {
            done = false;
            right[i].y -= 2;
          }
          if(right[i].y < (5 + (BOARD_SIZE+5)*i))
            right[i].y = (5 + (BOARD_SIZE+5)*i); 
         }
        if(left[i] != null) {
          if(left[i].y < (BASE - (BOARD_SIZE+5)*i)) {
            done = false;
            left[i].y += 2; 
          }
          if(left[i].y > (BASE - (BOARD_SIZE+5)*i))
            left[i].y = (BASE - (BOARD_SIZE+5)*i);

        }
      }

      return done;
    }

    private void drawBoards() {
      Graphics2D g = (Graphics2D) canvas.getImageGraphics();

      drawBoards(g);

      g.dispose();

      canvas.paintCanvas();
    }

    private void drawBoards(Graphics2D g) {
      for(int i = 0; i < boards.length; i++)
        if(boards[i] != null)
          boards[i].draw(g); 

    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.setContent("We start with the left-hand side which is the number of ways to tile an (m+n)-board. If m=4 and n=2, this is");
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSubscript("f","4+2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("13")},
            20f));
      }
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        details.append("The orange line shows whether a board is breakable at cell 4. If it is breakable, the line is solid; if it is unbreakable, the line is dashed.",SET_MARK|SCROLL);

        for(int i = 0; i < boards.length; i++)
          boards[i].breakHighlight(4,true);

        drawBoards();
      }
    }    

    private class Part2 implements AnimationPart {
      public void animate() {
        details.append("To count the right-hand side, we will first look at the boards which are breakable at 4. It is clear that the tiling can be decomposed into an 4-tiling followed by an 2-tiling. Notice that there are f\u2084\u00a0=\u00a05 possible 4-tilings and  f\u2082\u00a0=\u00a02 possible 2-tilings. These tilings are independent, so we multiply them to produce the first term on the right-hand side.",NEWLINES|SET_MARK|SCROLL);
        
        boards[0].highlight(true);
        boards[1].highlight(true);
        boards[2].highlight(true);
        boards[3].highlight(true);
        boards[6].highlight(true);

        boards[5].highlight(true);
        boards[8].highlight(true);
        boards[10].highlight(true);
        boards[11].highlight(true);
        boards[12].highlight(true); 

        drawBoards();
      }
    }

    private class Part3 implements AnimationPart {
      public void animate() {
        details.append("The total number of breakable tilings is",
                       NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSubscript("f","4"),
              MathSpace.getNoSpace(),
              MathSymbol.getSubscript("f","2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("5 \u00d7 2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("10")},20f),SCROLL); 

        for(float x = boards[0].x; x <= RIGHT_SIDE; x += 2) {
          boards[0].x = x;
          boards[1].x = x;
          boards[2].x = x;
          boards[3].x = x;
          boards[6].x = x;
          drawBoards();
          sleep();
        }

        boards[0].x = RIGHT_SIDE;
        boards[1].x = RIGHT_SIDE;
        boards[2].x = RIGHT_SIDE;
        boards[3].x = RIGHT_SIDE;
        boards[6].x = RIGHT_SIDE;
        drawBoards();
        sleep();

        while(!moveBoards()) {
          drawBoards();
          sleep();
        }
        drawBoards();

        for(float x = boards[5].x; x <= RIGHT_SIDE; x += 2) {
          boards[5].x = x;
          boards[8].x = x;
          boards[10].x = x;
          boards[11].x = x;
          boards[12].x = x;
          drawBoards();
          sleep();
        }

        boards[5].x = RIGHT_SIDE;
        boards[8].x = RIGHT_SIDE;
        boards[10].x = RIGHT_SIDE;
        boards[11].x = RIGHT_SIDE;
        boards[12].x = RIGHT_SIDE;
        drawBoards();
        sleep();

        while(!moveBoards()) {
          drawBoards();
          sleep();
        }
        drawBoards();    
      }
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        details.append("Now we will count the unbreakable tilings. Since 4-1 = 3, there are f\u2083 = 3 ways to tile the head, and since 2-1 = 1, there is only f\u2081 = 1 way to tile the tail.",SET_MARK|SCROLL);

        boards[0].highlight(false);
        boards[1].highlight(false);
        boards[2].highlight(false);
        boards[3].highlight(false);
        boards[6].highlight(false);

        boards[5].highlight(false);
        boards[8].highlight(false);
        boards[10].highlight(false);
        boards[11].highlight(false);
        boards[12].highlight(false);

        boards[4].highlight(true);
        boards[7].highlight(true);
        boards[9].highlight(true);
        drawBoards();
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        details.append("So, the total number of unbreakable tilings is",
                       NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSubscript("f","3"),
              MathSpace.getNoSpace(),
              MathSymbol.getSubscript("f","1"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("3 \u00d7 1"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("3")},20f),SCROLL);

        for(float x = boards[4].x; x <= RIGHT_SIDE; x += 2) {
          boards[4].x = x;
          boards[7].x = x;
          boards[9].x = x;
          drawBoards();
          sleep();
        }

        boards[4].x = RIGHT_SIDE;
        boards[7].x = RIGHT_SIDE;
        boards[9].x = RIGHT_SIDE;
        drawBoards();        
      }
    }

    private class Part6 implements AnimationPart {
      public void animate() {
        boards[4].highlight(false);
        boards[7].highlight(false);
        boards[9].highlight(false); 
        drawBoards();

        details.append("It is clear that the theorem holds in this example,",
                       SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSubscript("f","6"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("13"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSubscript("f","4"),
              MathSpace.getNoSpace(),
              MathSymbol.getSubscript("f","2"),
              MathSymbol.getTextSymbol("+"),
              MathSymbol.getSubscript("f","3"),
              MathSpace.getNoSpace(),
              MathSymbol.getSubscript("f","1"),},
            20f),SCROLL);

      }
    }
  }  
}
