package modules.oresme;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.font.*;

import vcp.animation.*;
import vcp.utilities.*;
import vcp.mathnotation.*;

import info.clearthought.layout.TableLayout;

public class OresmeVisualization extends JPanel 
  implements 
    vcp.utilities.ScrollableTextAreaConstants {
  
  AnimationCanvas canvas;

  OresmeAnimation ani;

  ControlPanel cp;

  ScrollableTextArea details;

  public OresmeVisualization() {
    super();

    //setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    setLayout(new TableLayout(new double[][] 
      {{TableLayout.FILL},
       {TableLayout.PREFERRED,
        TableLayout.PREFERRED,
        TableLayout.FILL,
        //TableLayout.PREFERRED,
        TableLayout.PREFERRED}
      }));
    
    cp = new SliderControlPanel();
    
    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));

    //we want some margins around the text
    details.setMargins(6);

    canvas = new AnimationCanvas(getBackground());
    //canvas = new AnimationCanvas(Color.orange);

    MathExpression expr = new MathExpression
      (new MathSymbol[]
        {MathSymbol.getSummationSymbol
         ("i = 1", "\u221e",
          MathSymbol.getFractionSymbol
          (MathSymbol.getTextSymbol("i").toMathExpression(),
           MathSymbol.getSuperscript("2","i").toMathExpression()),
          "Sum of i/2^i as i goes from 1 to infinity"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getTextSymbol("2")
        },25);

    JLabel theorem = new JLabel
      (new ImageIcon(expr.toImage(10)));

    add(theorem,"0,0,c,c");     
    add(new JScrollPane(canvas),"0,1,C,C");
    add(details.getJScrollPane(),"0,2");
    add(cp,"0,3");

  }

  public void addNotify() {
    super.addNotify();

    //don't create the animation until the visualization has been
    //added to the main window
    ani = new OresmeAnimation();
  }

  protected class OresmeAnimation extends FPSAnimation {
    UnitPart s1;
    UnitPart s2pre;
    UnitPart[] s2;
    UnitPart[] s2Copy;

    GraphicLabel[] fractions;

    DividerLine[] lines;

    GlyphVector[] oneLabel;

    public OresmeAnimation() {
      super(cp,60);
      //super(cp, 100, 0, 10)

      setPartArray(createPartArray());

      Graphics2D g2d = canvas.getImageGraphics();

      s1 = new UnitPart("S1",1,false);
      s1.draw(g2d,30,330);

      s2pre = new UnitPart("S2",1,false);
      s2pre.draw(g2d,320,330);

      s2 = null;
      s2Copy = null;

      oneLabel = new GlyphVector[] {
        (new Font("Serif",Font.BOLD, 12)).createGlyphVector
        (g2d.getFontRenderContext(), "1"),
        (new Font("Serif",Font.BOLD, 12)).createGlyphVector
        (g2d.getFontRenderContext(), "1"),
        (new Font("Serif",Font.BOLD, 12)).createGlyphVector
        (g2d.getFontRenderContext(), "1"),
        (new Font("Serif",Font.BOLD, 12)).createGlyphVector
        (g2d.getFontRenderContext(), "1"),
      };

      drawBlocks();
      canvas.paintCanvas();
    }

    protected AnimationPart[] createPartArray() {
      return new AnimationPart[] {
        new Part0(),
        new Part1(),
        new Part2(),
        new Part3(),
        new Part4(),
        new Part5(),
        new Part6()
      };
    }

    private void drawBlocks() {
      Graphics2D g;
      drawBlocks(g = canvas.getImageGraphics());
      g.dispose();
    }

    private void drawBlocks(Graphics2D g) {
      if(oneLabel != null && s2pre != null) {
        float w = (float)oneLabel[0].getLogicalBounds().getWidth();
        float h = (float)oneLabel[0].getLogicalBounds().getHeight();

        g.drawGlyphVector(oneLabel[0],s1.x - w - 5, s1.y + 25 + h/2f);
        g.drawGlyphVector(oneLabel[1],s1.x + s1.getWidth()/2f - w, s1.y+50+h+2);
        g.drawGlyphVector(oneLabel[2],s2pre.x - w - 5, s2pre.y + 25 + h/2f);
        g.drawGlyphVector(oneLabel[3],s2pre.x + s2pre.getWidth()/2f - w, 
                          s2pre.y+50+h+2);
      }

      if(lines != null)
        for(int i = 0; i < lines.length; i++)
          lines[i].draw(g);

      s1.draw(g);
      
      if(s2pre != null)
        s2pre.draw(g);

      if(s2 != null)
        for(int i = 0; i < s2.length; i++)
          s2[i].draw(g);

      if(s2Copy != null) 
        for(int i = 0; i < s2.length; i++)
          s2Copy[i].draw(g);

      if(fractions != null)
        for(int i = 0; i < fractions.length; i++)
          fractions[i].draw(g);

    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.setContent("We start by visually representing the number 2 as two unit squares. We will stretch the squares horizontally to make the visualization more clear.");        
      }
    }
    
    private class Part1 implements AnimationPart {
      public void animate() {
        Graphics2D g;

        boolean s1result = false, s2result = false;

        while(!s1result || !s2result) {
          s1result = s1.expand();
          s2result = s2pre.expand();
          drawBlocks(g = canvas.getImageGraphics());
          g.dispose();
          canvas.paintCanvas();
          sleep();     
        }

      details.setContent("Next, we cut each square in half vertically, then the right section in half, then the new right-most section in half, and so on to infinity.");
      }
    }

    private class Part2 implements AnimationPart {
      public void animate() {
        oneLabel = null;

        Graphics2D g = canvas.getImageGraphics();
        float alpha;
        
        s2 = new UnitPart[] {
          new UnitPart("\u03b1",2),
          new UnitPart("\u03b2",4),
          new UnitPart("\u03b3",8),
          new UnitPart("\u03b4",16),
          new UnitPart(null,32),
          new UnitPart(null,32)
        };
        
        s2Copy = new UnitPart[] {
          new UnitPart("\u03b1",2),
          new UnitPart("\u03b2",4),
          new UnitPart("\u03b3",8),
          new UnitPart("\u03b4",16),
          new UnitPart(null,32),
          new UnitPart(null,32)
        };
        
        float x = 320;
        for(int i = 0; i < s2.length; i++) {
          s2[i].setAlpha(0);
          s2Copy[i].setAlpha(0);
          s2[i].draw(g,x,330);
          s2Copy[i].draw(g,x,330);
          
          x += s2[i].getWidth();
        }
        g.dispose();

        lines = new DividerLine[5];
        x = s1.x;
        float y = s1.y;
        for(int i = 0; i < lines.length; i++) {
          x += s2[i].getWidth();
          lines[i] = new DividerLine(x,y,x,y+s1.getHeight());
        }

        for(alpha = 0; alpha <= 1; alpha += .05) {
          for(int i = 0; i < s2.length; i++)
            s2[i].setAlpha(alpha);
          
          for(int i = 0; i < lines.length; i++) {
            lines[i].setAlpha(alpha);
          }

          s1.setLabelAlpha(1-alpha);

          s2pre.setAlpha(1-alpha);
          drawBlocks(g = canvas.getImageGraphics());
          g.dispose();
          canvas.paintCanvas();
          sleep();
        }
        
        s1.setLabel(null);
        s2pre = null;

        drawBlocks();
        canvas.paintCanvas();

        details.setContent("Now, we will stack the sections of the second square on top of the first square. Note that this process should go on to infinity, but we truncate it by simply fading out the right-most visually represented section.");
      }
    }
    
    private class Part3 implements AnimationPart {
      public void animate() {
        Graphics2D g = canvas.getImageGraphics();
        float alpha;

        int i = 0;
        for(; i < s2.length - 2; i++) {
          s2Copy[i].setAlpha(.3f);
          
          while(!s2[i].move(s1.x+s1.getWidth()-s2[i].getWidth(),
                            s1.y-(50*(i+1)))
                ) {
            s2[i].draw(g = canvas.getImageGraphics());
            drawBlocks(g = canvas.getImageGraphics());
            g.dispose();
            canvas.paintCanvas();
            sleep();
          }
          drawBlocks(g = canvas.getImageGraphics());

          for(float y = lines[4].y1; y >= s2[i].y; y-=4) {
            for(int j = i; j < lines.length; j++)
              lines[j].y1 = y;
            drawBlocks();
            canvas.paintCanvas();
            sleep();
          }

          g.dispose();
          canvas.paintCanvas();
          sleep();
        }
        alpha = 1f;
        
        s2Copy[i].setAlpha(.3f);
        s2Copy[i+1].setAlpha(.3f);
        while(!s2[i].move(s1.x+s1.getWidth()-s2[i].getWidth(),
                          s1.y-(50*(i+1)))
              ) {
          s2[i+1].setAlpha(alpha *= .85);
          drawBlocks(g = canvas.getImageGraphics());
          g.dispose();
          canvas.paintCanvas();
          sleep();
        }
        drawBlocks(g = canvas.getImageGraphics());
        g.dispose();
        canvas.paintCanvas();
        sleep();

        for(alpha = .3f; alpha >= 0; alpha -= .01) {
          for(i = 0; i < s2Copy.length; i++)
            s2Copy[i].setAlpha(alpha);

          drawBlocks(g = canvas.getImageGraphics());
          g.dispose();
          canvas.paintCanvas();
          sleep();
        }

        s2Copy = null;

        int step = 2;
        int toMove = (int)(175f - s1.x);
        for(int x = 0; x <= toMove; x += step) {
          s1.x += step;
          for(i = 0; i < s2.length; i++)
            s2[i].x += step;
          for(i = 0; i < lines.length; i++)
            lines[i].x1 = lines[i].x2 += step;

          drawBlocks(g = canvas.getImageGraphics());
          g.dispose();
          canvas.paintCanvas();
          sleep();
        }
      }  
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        s1.setLabel(null);

        Graphics2D g = canvas.getImageGraphics();

        MathExpression m = new MathExpression
          (new MathSymbol[] 
            {MathSymbol.getFractionSymbol("1","2")},
           12);
        float size = (float)((12f*20f)/m.getHeight(g));

        MathExpression half = new MathExpression
          (new MathSymbol[] 
            {MathSymbol.getFractionSymbol("1","2")},
           size);
        MathExpression fourth = new MathExpression
          (new MathSymbol[] 
          {MathSymbol.getFractionSymbol("1","4")},
           size);
        MathExpression eighth = new MathExpression
          (new MathSymbol[] 
          {MathSymbol.getFractionSymbol("1","8")},
           size);
        MathExpression sixteenth = new MathExpression
          (new MathSymbol[] 
          {MathSymbol.getFractionSymbol("1","16")},
           size);

        fractions = new GraphicLabel[] {
          new GraphicLabel(half),
          new GraphicLabel(fourth),
          new GraphicLabel(eighth),
          new GraphicLabel(sixteenth),

          new GraphicLabel(fourth),
          new GraphicLabel(eighth),
          new GraphicLabel(sixteenth),

          new GraphicLabel(eighth),
          new GraphicLabel(sixteenth),

          new GraphicLabel(sixteenth),
        };

        int f = 0;
        for(int i = 0; i < 4; i++) {
          for(int j = i; j < 4; j++, f++) {
            fractions[f].x = (float)
              (s2[j].x - 
               (s2[j].getWidth()/2f + fractions[f].expr.getWidth(g)/2f));
            fractions[f].y = (float) (s1.y - ((i-1)*50) - 25 + 
                                      fractions[f].expr.getHeight(g)/2f);
          }
        }

        for(float alpha = 1; alpha >= 0; alpha -= .05) {
          for(int i = 0; i < s2.length; i++)
            s2[i].setLabelAlpha(alpha);
          for(int i = 0; i < fractions.length; i++)
            fractions[i].setAlpha(1-alpha);
          
          drawBlocks();
          canvas.paintCanvas();
          sleep();
        }


        for(int i = 0; i < s2.length; i++)
          s2[i].setLabel(null);
        for(int i = 0; i < fractions.length; i++)
          fractions[i].setAlpha(1);
        
        drawBlocks();
        canvas.paintCanvas();

        details.setContent("Notice what we have created.  From two unit squares we now have a stair-step structure which represents");

        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getFractionSymbol("1","2"),
              MathSymbol.getTextSymbol("+"),
              MathSymbol.getFractionSymbol("2","4"),
              MathSymbol.getTextSymbol("+"),
              MathSymbol.getFractionSymbol("3","8"),
              MathSymbol.getTextSymbol("+"),
              MathSymbol.getFractionSymbol("4","16"),
              MathSymbol.getTextSymbol("+"),
              MathSymbol.getFractionSymbol("5","32"),
              MathSymbol.getTextSymbol("+"),
              MathSymbol.getTextSymbol("...")},20));

        details.append("which equals");

        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSummationSymbol
              ("i = 1", "\u221e",
               MathSymbol.getFractionSymbol
               (MathSymbol.getTextSymbol("i").toMathExpression(),
                MathSymbol.getSuperscript("2","i").toMathExpression()),
               "Sum of i/2^i as i goes from 1 to infinity")
             },30));
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        details.setContent("Now we will emphasize the fact that this stair-step structure is equal to 2 unit squares. Note again that this process would continue to infinity, but in the visualization, we represent this continuation by filling in the top unit square with a lighter-colored block section.",SET_MARK|SCROLL);

        for(float alpha = 1; alpha >= 0; alpha -= .05) {
          for(int i = 0; i < fractions.length; i++)
            fractions[i].setAlpha(alpha);
          for(int i = 0; i < lines.length; i++)
            lines[i].setAlpha(alpha);

          drawBlocks();
          canvas.paintCanvas();
          sleep();
        }

        fractions = null;
        lines = null;

        drawBlocks();
        canvas.paintCanvas();

      }
    }

    private class Part6 implements AnimationPart {
      public void animate() {
        float x = s1.x;
        boolean res;
        for(int i = 0; i < s2.length; i++)
          s2[i].speed=2;
        for(int i = 0; i < s2.length-1; i++) {
          float newx = s2[i].x;
          res = false;
          while(!res) {
            res = true;
            for(int j = i; j < s2.length-1; j++)
              res = s2[j].move(s2[j].x,(s1.y-50 - 50*(j-i))) && res;

            drawBlocks();
            canvas.paintCanvas();
            sleep();
          }

          sleep(10);

          res = false;
          while(!res) {
            res = s2[i].move(x,s2[i].y);
            drawBlocks();
            canvas.paintCanvas();
            sleep();
          }

          x = newx;
        }

        sleep(10);
        
        Rectangle2D.Float rect = new Rectangle2D.Float(x,s1.y-50,0,50);
        float width = s1.x+250-x+1;
        Graphics2D g;
        for(rect.width = 0; rect.width <= width; rect.width += 1) {
          g = canvas.getImageGraphics();
          g.setColor(Color.gray);
          g.setStroke(new BasicStroke(3));
          g.draw(rect);
          g.fill(rect);
          drawBlocks(g);
          g.dispose();
          canvas.paintCanvas();
          sleep();
        }

        details.append("So, we now have a 1\u00d72 block that has an area of 2.",NEWLINES|SET_MARK|SCROLL);
      }
    }
  }
  
}
