package modules.subsets;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.border.LineBorder;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.RenderingHints;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Arc2D;
import java.awt.geom.Line2D;

import vcp.utilities.ScrollableTextArea;
import vcp.utilities.ExtendedTableLayout;
import vcp.utilities.VerticalJLabel;
import vcp.mathnotation.*;
import vcp.animation.*;

import info.clearthought.layout.TableLayout;

public class SubsetsVisualization extends JPanel 
  implements 
    vcp.utilities.ScrollableTextAreaConstants {
  
  static final String ON = "\u2022";
  static final String OFF = "";


  private ScrollableTextArea details;

  SubsetsPanel sp;

  public SubsetsVisualization() {
    setLayout(new TableLayout
              (new double[][]
                {{TableLayout.FILL},
                 {TableLayout.PREFERRED,
                  TableLayout.FILL,
                  TableLayout.PREFERRED,
                  TableLayout.PREFERRED}}));


    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));
    details.setMargins(6);

    add(new JLabel(new ImageIcon
                   (new MathExpression
                    (new MathSymbol[] 
                      {MathSymbol.getSummationSymbol
                       ("r = 0","n",
                        MathSymbol.getBinomialCoefficient("n","r"),
                        "Sum of C(n,r) with r = {0,...,n}" ),
                       MathSymbol.getTextSymbol("="),
                       MathSymbol.getSuperscript("2", "n")},20).toImage(5))),
        "0,0,l,b");
                    
    add(details.getJScrollPane(),"0,2");

    ControlPanel cp = new SliderControlPanel();
    add(sp = new SubsetsPanel(),"0,1");
    add(cp,"0,3");

    SubsetsAnimation ani = new SubsetsAnimation(cp);
  }

  private class SubsetsPanel extends JPanel {
    JLabel[][] onoff;
    JLabel[] committee;
    JLabel[] combinations;
    JLabel[] students;
    ExtendedTableLayout t;
    int lineIndex = 1;
    
    boolean peopleLines = false;
    
    public SubsetsPanel() {
      double[] rows = new double[16];
      double[] columns = new double[6];
      
      for(int i = 0; i < rows.length; i++)
        rows[i] = TableLayout.PREFERRED;
      for(int i = 0; i < columns.length; i++)
        columns[i] = TableLayout.PREFERRED;
      
      setLayout(t = new ExtendedTableLayout(columns,rows));
      
      t.setHGap(8);
      t.setVGap(6);


      onoff = new JLabel[][] {
        {new JLabel(OFF), new JLabel(OFF), new JLabel(OFF), new JLabel(OFF)},
        
        {new JLabel(ON), new JLabel(OFF), new JLabel(OFF), new JLabel(OFF)},
        {new JLabel(OFF), new JLabel(ON), new JLabel(OFF), new JLabel(OFF)},
        {new JLabel(OFF), new JLabel(OFF), new JLabel(ON), new JLabel(OFF)},
        {new JLabel(OFF), new JLabel(OFF), new JLabel(OFF), new JLabel(ON)},
        
        {new JLabel(ON), new JLabel(ON), new JLabel(OFF), new JLabel(OFF)},
        {new JLabel(ON), new JLabel(OFF), new JLabel(ON), new JLabel(OFF)},
        {new JLabel(ON), new JLabel(OFF), new JLabel(OFF), new JLabel(ON)},
        {new JLabel(OFF), new JLabel(OFF), new JLabel(ON), new JLabel(ON)},
        {new JLabel(OFF), new JLabel(ON), new JLabel(OFF), new JLabel(ON)},
        {new JLabel(OFF), new JLabel(ON), new JLabel(ON), new JLabel(OFF)},
        
        {new JLabel(ON), new JLabel(ON), new JLabel(ON), new JLabel(OFF)},
        {new JLabel(ON), new JLabel(ON), new JLabel(OFF), new JLabel(ON)},
        {new JLabel(ON), new JLabel(OFF), new JLabel(ON), new JLabel(ON)},
        {new JLabel(OFF), new JLabel(ON), new JLabel(ON), new JLabel(ON)},
        
        {new JLabel(ON), new JLabel(ON), new JLabel(ON), new JLabel(ON)}
      };
      
      committee = new JLabel[16];
      
      for(int i = 0; i < 16; i++) {
        for(int j = 0; j < 4; j++) {
          add(onoff[i][j], j+2 + "," + i + ",c,c");
          onoff[i][j].setVisible(false);
        }
        add(committee[i] = new JLabel(generateCommittee(onoff[i])),
            "1," + i);
        committee[i].setVisible(false);
      }

      combinations = new JLabel[5];
      String[] totals = new String[] {"1","4","6","4","1"};
      for(int i = 0; i < 5; i++)
        combinations[i] = new JLabel
          (new ImageIcon
           (new MathExpression
            (new MathSymbol[]
              {MathSymbol.getBinomialCoefficient("4",i+""),
               MathSymbol.getTextSymbol("="),
               MathSymbol.getTextSymbol(totals[i])
              },20f).toImage(2)));

      add(combinations[0],"0,0,c,c");
      add(combinations[1],"0,1,0,4,c,c");
      add(combinations[2],"0,5,0,10,c,c");
      add(combinations[3],"0,11,0,14,c,c");
      add(combinations[4],"0,15,c,c");
      for(int i = 0; i < 5; i++)
        combinations[i].setVisible(false);

      t.insertRow(0,TableLayout.PREFERRED);

      add(new JLabel("Group Members"),"1,0,l,b");

      students = new JLabel[4];
      add(students[0] = new VerticalJLabel("Nick"),"2,0");
      add(students[1] = new VerticalJLabel("Shalanah"),"3,0");
      add(students[2] = new VerticalJLabel("Eric"),"4,0");
      add(students[3] = new VerticalJLabel("Allison"),"5,0");
      for(int i = 0; i < 4; i++)
        students[i].setVisible(false);
    }

    public void paintComponent(Graphics g) {
      super.paintComponent(g);
      
      Graphics2D g2d = (Graphics2D) g;
      
      g2d.setColor(Color.BLACK);
      g2d.setStroke(new BasicStroke(1));
      g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                           RenderingHints.VALUE_ANTIALIAS_ON);      
      int[] lines = new int[] {0,1,6,12,16,17};
      
      double y =  -(double)(t.getVGap())/2.0;
      int i = 0;
      for(int j = 0; j < lineIndex; j++) {
        y += t.getRowPixels(i++) + t.getVGap();
        for(; i < lines[j]; i++)
          y += t.getRowPixels(i) + t.getVGap();
        g2d.draw
          (new Line2D.Double(0,y,getWidth(),y));
      }

      if(peopleLines) {
        int j = 0;
        double x = -(t.getHGap()/2.0);
        for(; j < 1; j++)
          x += t.getColumnPixels(j) + t.getHGap();
        
        for(; j < 6; j++) {
          x += t.getColumnPixels(j) + t.getHGap();
          g2d.draw
            (new Line2D.Double(x,0,x,getHeight()));
        }
      }
    }
    
    private String generateCommittee(JLabel[] l) {
      StringBuffer sb = new StringBuffer("{");
      
      if(l[0].getText().equals(ON))
        sb.append("Nick, ");
      if(l[1].getText().equals(ON))
        sb.append("Shalanah, ");
      if(l[2].getText().equals(ON))
        sb.append("Eric, ");
      if(l[3].getText().equals(ON))
        sb.append("Allison, ");
      
      
      if(sb.length() > 1)
        return sb.toString().substring(0,sb.length()-2) + "}";
      else
        return "{ }";
    }
    

  }

  private class SubsetsAnimation extends Animation {
    public SubsetsAnimation(ControlPanel c) {
      super(c, 500);
      setPartArray(new AnimationPart[]
        {
          new Part0(),
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5(),
          new Part6(),
          new Part7(),
          new Part8(),
          new Part9(),
        });

      //part = 3;
    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.setContent("Four students (n=4) are planning to work on a linear algebra project. We are counting the number of possible project groups with Nick, Shalanah, Eric, and Allison as the potential project members. Notice that we are also counting the number of subsets of a set of size 4.\n\nThese possibilities are first counted using the left-hand side of the identity. The possible groups of students are listed on the left according to their size.");
      }
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        details.append("There is 1 way that no one can work on the project. It is the empty set.",NEWLINES|SET_MARK|SCROLL);
        sp.committee[0].setVisible(true);
        sp.combinations[0].setVisible(true);
        sp.lineIndex++;
        sp.repaint();
      }
    }

    private class Part2 implements AnimationPart {
      public void animate() {
        details.append("There are 4 ways that just 1 student can work on the project.",NEWLINES|SET_MARK|SCROLL);
        sp.committee[1].setVisible(true);
        sp.committee[2].setVisible(true);
        sp.committee[3].setVisible(true);
        sp.committee[4].setVisible(true);
        sp.combinations[1].setVisible(true);
        sp.lineIndex++;
        sp.repaint();
      }
    }

    private class Part3 implements AnimationPart {
      public void animate() {
        details.append("There are 6 ways that a pair of students can work on the project.",NEWLINES|SET_MARK|SCROLL);
        sp.committee[5].setVisible(true);
        sp.committee[6].setVisible(true);
        sp.committee[7].setVisible(true);
        sp.committee[8].setVisible(true);
        sp.committee[9].setVisible(true);
        sp.committee[10].setVisible(true);
        sp.combinations[2].setVisible(true);
        sp.lineIndex++;
        sp.repaint();
      }
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        details.append("There are 4 ways that a group of 3 students can work on the project.",NEWLINES|SET_MARK|SCROLL);
        sp.committee[11].setVisible(true);
        sp.committee[12].setVisible(true);
        sp.committee[13].setVisible(true);
        sp.committee[14].setVisible(true);
        sp.combinations[3].setVisible(true);
        sp.lineIndex++;
        sp.repaint();
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        details.append("There is only 1 way that all 4 students can work on the project together.",NEWLINES|SET_MARK|SCROLL);
        sp.committee[15].setVisible(true);
        sp.combinations[4].setVisible(true);
        sp.lineIndex++;
        sp.repaint();
      }
    }

    private class Part6 implements AnimationPart {
      public void animate() {
        details.append("Thus, there are a total of",NEWLINES|SET_MARK);
        details.append(MathSymbol.getTextSymbol
                       ("1 + 4 + 6 + 4 + 1 = 16").toMathExpression(18f));

        details.append("subsets that can be formed from these 4 students.",SCROLL);
      }
    }

    private class Part7 implements AnimationPart {
      public void animate() {
        for(int i = 0; i < 4; i++)
          sp.students[i].setVisible(true);

        sp.peopleLines = true;
        sp.repaint();

        details.append("Now we will count this number using the right-hand side of the theorem based on which students work on the project. For each listed group, a highlighted dot appears under the students' name if that student is working on the project.",NEWLINES|SET_MARK|SCROLL);
      }
    }

    public void colorRow(int i, Color c) {
      sp.committee[i].setForeground(c);
      for(int j = 0; j < 4; j++)
        if(sp.onoff[i][j].getText().equals(ON)) {
          sp.onoff[i][j].setForeground(c);
          sp.students[j].setForeground(c);
        }
    }

    private class Part8 implements AnimationPart {
      public void animate() {
        for(int i = 0; i < sp.onoff.length; i++) {
          if(i > 0)
            colorRow(i-1,Color.black);
          colorRow(i,Color.blue);
          for(int j = 0; j < 4; j++)
            sp.onoff[i][j].setVisible(true);
          sleep();
        }
        
        colorRow(15,Color.black);

        details.append("Each student either does or does not work on the project. Since those are the only 2 options for each of the 4 students, there are a total of",NEWLINES|SET_MARK);
        details.append
          (MathSymbol.getTextSymbol
           ("2 \u00d7 2 \u00d7 2 \u00d7 2 = 16").toMathExpression(18f));
        details.append("different project groups that can be chosen from these 4 students.",SCROLL);
      }
    }

    private class Part9 implements AnimationPart {
      public void animate() {
        details.append("It is clear in this example that the theorem holds true,",NEWLINES|SET_MARK);
        details.append(new MathExpression
                       (new MathSymbol[] 
                         {MathSymbol.getSummationSymbol
                          ("r = 0","4",
                           MathSymbol.getBinomialCoefficient("4","r"),
                           "Sum of C(4,r) with r = {0,...,4}" ),
                          MathSymbol.getTextSymbol("="),
                          MathSymbol.getSuperscript("2", "4")},18),SCROLL);
      }
    }        
  }
  
}
