package vcp.animation;

/**
 * This is a GUI element used to control an {@link Animation}. 
 *
 * @author phil kaasa
 * @version 1.0
 */
public abstract class ControlPanel extends javax.swing.JPanel {
  /** the animation this ControlPanel controls */
  Animation ani;

  /**
   * display the given speed in this control panel
   *
   * @param s the speed to display
   */
  public abstract void showSpeed(double s);


  /**
   * Called when a part of an animation begins. This method is
   * provided so GUI components can be disabled or names changed,
   * etc. if need be.
   */
  public abstract void partStarted();
  
  /**
   * Called when a part of an animation finishes. This method is
   * provided so GUI components can be enabled or names changed,
   * etc. if need be.
   */
  public abstract void partFinished();

  /**
   * Called when the animation associated with this ControlPanel is
   * completed.
   */
  public abstract void animationComplete();

  /**
   * Creates a new <code>ControlPanel</code> instance.
   */
  public ControlPanel() {
    super();
  }

  /**
   * Creates a new <code>ControlPanel</code> instance. This
   * <code>ControlPanel</code> will control the passed Animation.
   *
   * @param a the Animation to control
   */
  public ControlPanel(Animation a) {
    setAnimation(a);
  }

  /**
   * Set the animation for this control panel to control.
   *
   * @param a an <code>Animation</code> value
   */
  public void setAnimation(Animation a) {
    ani = a;
    showSpeed(ani.getSpeed());
  }

}