package vcp.animation;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * An animation control panel. Displays current animation speed and
 * has buttons to speed up, slow down, pause, and continue animation.
 *
 * @author phil kaasa
 * @version 1.0
 * @see Animation
 * @see FPSAnimation
 */
public class DefaultControlPanel extends ControlPanel 
  implements ActionListener {

  /** icon for pause button */
  public static ImageIcon PAUSE_ICON = 
    new ImageIcon(DefaultControlPanel.class.getResource
		  ("/vcp/animation/media-pause.png"));
  /** icon for play button */
  public static ImageIcon PLAY_ICON = 
    new ImageIcon(DefaultControlPanel.class.getResource
		  ("/vcp/animation/media-play.png"));

  /** icon for speed up button */
  public static ImageIcon FFWD_ICON = 
    new ImageIcon(DefaultControlPanel.class.getResource
		  ("/vcp/animation/media-ffwd.png"));

  /** icon for slow down button */
  public static ImageIcon REWIND_ICON = 
    new ImageIcon(DefaultControlPanel.class.getResource
		  ("/vcp/animation/media-rewind.png"));

  /** button to continue animation to the next part */
  JButton nextPart;

  /** button to slow animation down */
  JButton slower;

  /** button to speed animation up */
  JButton faster;

  /** button to pause animation */
  JButton pause;

  /** label for current speed */
  JLabel speed;


  /**
   * Creates a new <code>DefaultControlPanel</code> with no action listener.
   */
  public DefaultControlPanel() {
    super();
    //super(new GridLayout(1,4));
    setLayout(new GridLayout(1,4));

    nextPart = new JButton("Start Visualization");
    slower = new JButton(REWIND_ICON);
    pause = new JButton(PAUSE_ICON);
    faster = new JButton(FFWD_ICON);
    speed = new JLabel(""); 

    nextPart.addActionListener(this);
    slower.addActionListener(this);
    pause.addActionListener(this);
    faster.addActionListener(this);

    add(speed);
    add(slower);
    add(pause);
    add(faster);
    add(nextPart);
  }

  public void addNotify() {
    super.addNotify();

    validate();
  }

  public void showSpeed(double s) {
    if(s == 0)
      s = 1;
    speed.setText("Speed: " + (int)s +"%");
  }
  
  public void partStarted() {
    nextPart.setText("Continue");
    nextPart.setEnabled(false); 
  }

  public void partFinished() {
    nextPart.setEnabled(true);
  }

  public void animationComplete() {
    nextPart.setEnabled(false);
    nextPart.setText("Done");
  }

  public void actionPerformed(ActionEvent e) {
    Object button = e.getSource();

    if(button == nextPart)
      //start the thread for the next part of the animation
      ani.startNextPart();

    else if(button == slower) {
      if(!ani.isPaused()) {
	//if not paused, decrease the speed
        ani.slower();
	showSpeed(ani.getSpeed());
      }
    }

    else if(button == faster)
      if(!ani.isPaused()) {
	//if not paused, increase the speed
        ani.faster();
	showSpeed(ani.getSpeed());
      } else {
	//if paused, step the animation
	if(ani.isInProgress())
	  ani.step();
      }
    
    else if(button == pause) {
      if(!ani.isPaused()) {
	//if not paused, pause
	pause.setIcon(PLAY_ICON);
        //cPanel.pause.setText("\u25ba");
        slower.setEnabled(false);
	faster.setIcon(null);
        faster.setText("Step");
        ani.pause();
      } else {
	//if paused, restart
        pause.setIcon(PAUSE_ICON);
        slower.setEnabled(true);
        faster.setIcon(FFWD_ICON);
	faster.setText("");
        ani.start();
      }
    }

  }
}