package vcp.animation;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.ImageIcon;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.util.Hashtable;

import info.clearthought.layout.TableLayout;

/**
 * An animation control panel. Displays current animation speed and
 * has buttons to speed up, slow down, pause, and continue animation.
 *
 * @author phil kaasa
 * @version 1.0
 * @see Animation
 * @see FPSAnimation
 */
public class SliderControlPanel extends ControlPanel 
  implements ActionListener,ChangeListener {

  /** icon for pause button */
  public static ImageIcon PAUSE_ICON = 
    new ImageIcon(SliderControlPanel.class.getResource
		  ("/vcp/animation/media-pause.png"));
  /** icon for play button */
  public static ImageIcon PLAY_ICON = 
    new ImageIcon(SliderControlPanel.class.getResource
		  ("/vcp/animation/media-play.png"));

  /** button to continue animation to the next part */
  JButton nextPart;

  /** button to speed animation up */
  JButton step;

  /** button to pause animation */
  JButton pause;

  /** slider to control the animation speed */
  JSlider slider;


  /**
   * Creates a new <code>SliderControlPanel</code> with no action listener.
   */
  public SliderControlPanel() {
    super();

    setLayout(new TableLayout(new double[][]
      {{TableLayout.PREFERRED,
        TableLayout.PREFERRED,
        TableLayout.PREFERRED,
        TableLayout.FILL,
        TableLayout.PREFERRED},
       {TableLayout.PREFERRED}}));
         

    //super();
    //setLayout(new GridLayout(1,4));

    nextPart = new JButton("Start Visualization");
    slider = new JSlider();
    pause = new JButton(PAUSE_ICON);
    step = new JButton("Step");

    step.setEnabled(false);

    Hashtable labelTable = new Hashtable();

    labelTable.put(new Integer(0), new JLabel("Slow"));
    labelTable.put(new Integer(100), new JLabel("Fast"));
    slider.setLabelTable(labelTable);
    slider.setPaintLabels(true);

    nextPart.addActionListener(this);
    slider.addChangeListener(this);
    pause.addActionListener(this);
    step.addActionListener(this);

    add(slider,"0,0");
    add(pause,"1,0");
    add(step,"2,0");
    add(nextPart,"4,0");

    java.awt.Dimension d = nextPart.getPreferredSize();
    nextPart.setPreferredSize(d);
    nextPart.setMinimumSize(d);
    nextPart.setMaximumSize(d);
    nextPart.setSize(d);
  }

  public void addNotify() {
    super.addNotify();

    validate();
  }

  public void showSpeed(double s) {
    slider.setValue((int)s);
  }
  
  public void partStarted() {
    nextPart.setText("Continue");
    nextPart.setEnabled(false);
    
    if(ani.isPaused())
      step.setEnabled(true);
  }

  public void partFinished() {
    nextPart.setEnabled(true);
    step.setEnabled(false);
  }

  public void animationComplete() {
    nextPart.setEnabled(false);
    step.setEnabled(false);
    slider.setEnabled(false);
    pause.setEnabled(false);
    nextPart.setText("Done");
  }

  public void actionPerformed(ActionEvent e) {
    Object button = e.getSource();

    if(button == nextPart)
      //start the thread for the next part of the animation
      ani.startNextPart();

    else if(button == step) {
      if(ani.isPaused()) {
	//if paused, step the animation
	if(ani.isInProgress())
	  ani.step();
      }
    }
    
    else if(button == pause) {
      if(!ani.isPaused()) {
	//if not paused, pause
	pause.setIcon(PLAY_ICON);
        if(ani.isInProgress())
          step.setEnabled(true);
        ani.pause();
      } else {
	//if paused, restart
        pause.setIcon(PAUSE_ICON);
	step.setEnabled(false);
        ani.start();
      }
    }
  }

  public void stateChanged(ChangeEvent e) {
    if(!slider.getValueIsAdjusting()) {
      ani.setSpeed(slider.getValue());
      showSpeed(ani.getSpeed());
    }
  }
}