package modules.bibd;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.border.LineBorder;
import java.awt.Color;
import java.awt.Font;

import vcp.utilities.ScrollableTextArea;
import vcp.mathnotation.*;
import vcp.animation.*;

import info.clearthought.layout.TableLayout;

public class BIBDVisualization extends JPanel 
  implements 
    vcp.utilities.ScrollableTextAreaConstants {
  private MatrixPanel m;
  private JButton start;
  private JLabel partLabel;
  private ScrollableTextArea details;

  private BIBDAnimation ani;

  public BIBDVisualization() {
    setLayout(new TableLayout
              (new double[][]
                {{TableLayout.FILL},
                 {TableLayout.PREFERRED,
                  TableLayout.PREFERRED,
                  TableLayout.FILL,
                  TableLayout.PREFERRED}}));
    //setResizable(false);

    //set up the part title label
    MathExpression expr = 
      MathSymbol.getTextSymbol("bk = vr").toMathExpression(20f);
    partLabel = new JLabel(new ImageIcon(expr.toImage(10)));
    partLabel.setAlignmentX(CENTER_ALIGNMENT);
    add(partLabel,"0,0");

    //the labels for rows and columns
    JLabel[] rl = new JLabel[7];
    JLabel[] cl = new JLabel[14];

    //Create the row and column labels. The commented out lines do
    //subscripting via html instead of using the unicode subscript
    //characters.
    for(int i = 0; i < rl.length; i++) {
      //rowLabel[0][i] = new JLabel("<html>v<sub>" + i + "</sub></html>");
      rl[i] = new JLabel("v" + getSubscript(i));
    }
    for(int j = 0; j < cl.length; j++) {
      //colLabel[0][j] = new JLabel("<html>B<sub>" + (j+1)+ "</sub></html>");
      cl[j] = new JLabel("B" + getSubscript(j+1));
    }

    //a BIBD incidence matrix for a (7,14,6,3,2)-design
    double[][] matrix = 
      new double[][] {{1,1,1,1,1,1,0,0,0,0,0,0,0,0},
                      {1,1,0,0,0,0,1,1,1,1,0,0,0,0},
                      {1,1,0,0,0,0,0,0,0,0,1,1,1,1},
                      {0,0,1,1,0,0,1,1,0,0,1,1,0,0},
                      {0,0,1,1,0,0,0,0,1,1,0,0,1,1},
                      {0,0,0,0,1,1,1,0,1,0,1,0,1,0},
                      {0,0,0,0,1,1,0,1,0,1,0,1,0,1}};

    //create the matrix panel with brackets and the appropriate labels.
    m = new MatrixPanel(matrix,true,rl,cl);
    m.setAlignmentX(CENTER_ALIGNMENT);
    //we don't want the matrix panel to resize on us
    m.setMinimumSize(m.getPreferredSize());
    m.setPreferredSize(m.getPreferredSize());
    m.setMaximumSize(m.getPreferredSize());
    //JScrollPane
    add(new JScrollPane(m), "0,1,c,c");

    //create the scrolling text box which has explanatory text for the
    //visualization.
    details = new ScrollableTextArea();
    details.setPreferredSize(new java.awt.Dimension(20,100));

    //we want some margins around the text
    details.setMargins(6);
    add(details.getJScrollPane(),"0,2");

    //create the control panel
    ControlPanel cPanel = new SliderControlPanel();
    add(cPanel,"0,3");

    //create the animation class
    ani = new BIBDAnimation(cPanel);
  }

  private class BIBDAnimation extends Animation {
    //private int part;

    public BIBDAnimation(ControlPanel c) {
      super(c);

      setPartArray(createPartArray());

      //part = 1;
    }

    protected AnimationPart[] createPartArray() {
      return new AnimationPart[] {
    new Part0(),
    new Part1(),
    new Part2(),
    new Part3(),
    new Part4(),
    new Part5(),
    new Part6(),
    new Part7(),
    new Part8(),
    new Part9(),
      };
    }

    private class Part0 implements AnimationPart {
      public void animate() {
    details.setContent("The index matrix is given for the example design, which is (v, b, r, k, \u03bb) = (7, 14, 6, 3, 2).");       
      }
    }    


    /**
     * Sums the number of 1s in the columns (bk)
     */  
    private class Part1 implements AnimationPart {
      public void animate() {
    int counter = 0;
    int total = 0;
    
    details.append("Count the number of 1s in each of the columns. " +
                       "There are 3 1s in each of the 14 columns",
                       SET_MARK|SCROLL|NEWLINES);

    for(int j = 0; j < m.getColumns(); j++) {
      counter = 0;
      m.colorColumnLabel(0,j,Color.RED);
      m.colorColumnLabel(1,j,Color.RED);
      for(int i = 0; i < m.getRows(); i++) {
        if(m.getEntry(i,j) == 1) {
          m.colorEntry(i,j,Color.RED);
          m.setColumnLabel(1,j,++counter);
          m.setCornerLabel(++total);
          sleep(delay);
        }
      }
      m.colorColumn(j,Color.BLACK);       
      m.colorColumnLabel(0,j,Color.BLACK);
      m.colorColumnLabel(1,j,Color.BLACK);
    }
    sleep(delay*3);

    details.append(", so");
        details.append(MathSymbol.getTextSymbol
                       ("bk = 14 \u00d7 3 = 42").toMathExpression(16f),
                       SET_MARK|SCROLL);
      }
    }

    /**
     * Sums the number of 1s in the rows (vr)
     */
    private class Part2 implements AnimationPart {
      public void animate() {
    int total = 0;
    int counter = 0;

    for(int i = 0; i < m.getColumns(); i++)
      m.setColumnLabel(1,i,"");
    m.setCornerLabel("");
    details.append("Now, count the number of 1s in each of the rows. " +
                       "There are 6 1s in each of the 7 rows",SET_MARK|SCROLL);

    for(int i = 0; i < m.getRows(); i++) {
      counter = 0;
      m.colorRowLabel(0,i,Color.BLUE);
      m.colorRowLabel(1,i,Color.BLUE);
      for(int j = 0; j < m.getColumns(); j++) {
        if(m.getEntry(i,j) == 1) {
          m.colorEntry(i,j,Color.BLUE);
          m.setRowLabel(1,i,++counter);
          m.setCornerLabel(++total);
          sleep(delay);
        }
      }
      m.colorRow(i,Color.BLACK);
      m.colorRowLabel(0,i,Color.BLACK);
      m.colorRowLabel(1,i,Color.BLACK);
    }
    details.append(", so");
        details.append(MathSymbol.getTextSymbol
                       ("vr = 7 \u00d7 6 = 42").toMathExpression(16f),SCROLL);
      }
    }
    
    private class Part3 implements AnimationPart {
      public void animate() {
    details.append("It is easy to see that the first part of the theorem holds in this example.",SET_MARK|SCROLL);  
      }
    }    

    /**
     * Shows the row to be removed (v4)
     */
    private class Part4 implements AnimationPart {
      public void animate() {
    for(int i = 0; i < m.getRows(); i++)
      m.setRowLabel(1,i,"");
    m.setCornerLabel("");
    
    int counter = 0;
    int total = 0;
    
        MathExpression expr = 
          MathSymbol.getTextSymbol
          ("r(k \u2212 1) = \u03bb(v \u2212 1)").toMathExpression(20f);
        partLabel.setIcon
          (new ImageIcon
           (expr.toImage(10)));
    
    details.setContent("Choose a variety and remove the row that " +
           "corresponds to that variety. We will choose " +
           "v\u2084 for this example.");
    m.colorRow(4, Color.LIGHT_GRAY, true); 
      }     
    }
 
    /**
     * Shows the columns to be removed.
     */
    private class Part5 implements AnimationPart {
      public void animate() {
    details.append("Also remove all the columns that correspond to " +
           "a block which does not contain v\u2084. Note that " +
           "the remaining columns are the r=6 columns which " +
                       "contain v\u2084",NEWLINES|SET_MARK|SCROLL);
      
    int[] c = new int[] {0,1,4,5,6,7,10,11};

    for(int i = 0; i < c.length; i++) {
      m.colorColumn(c[i], Color.LIGHT_GRAY, true);
      sleep(delay);
    }
      }
    }

    /**
     * Removes row v4 and all the appropriate columns.
     */
    private class Part6 implements AnimationPart {
      public void animate() {
    details.setContent("The new matrix is 6\u00d76, and it " +
           "does not contain the row v\u2084 \n or the columns B\u2081, B\u2082, " +
           "B\u2085, B\u2086, B\u2087, B\u2088, B\u2081\u2081," +
           " or B\u2081\u2082.");

    m.removeRow(4,false);
    m.colorColumn(0, Color.LIGHT_GRAY, true);
    m.colorColumn(1, Color.LIGHT_GRAY, true);
    m.colorColumn(4, Color.LIGHT_GRAY, true);
    m.colorColumn(5, Color.LIGHT_GRAY, true);
    m.colorColumn(6, Color.LIGHT_GRAY, true);
    m.colorColumn(7, Color.LIGHT_GRAY, true);
    m.colorColumn(10, Color.LIGHT_GRAY, true);
    m.colorColumn(11, Color.LIGHT_GRAY, true);    
    sleep(delay*3);
    m.removeColumn(0, true);
    m.removeColumn(0, true);
    m.removeColumn(2, true);
    m.removeColumn(2, true);
    m.removeColumn(2, true);
    m.removeColumn(2, true);
    m.removeColumn(4, true);
    m.removeColumn(4, false);
      }
    }

    /**
     * Sums the number of 1s in the columns (r(k-1))
     */
    private class Part7 implements AnimationPart {
      public void animate() {
    details.setContent("Now, we will count 1s the same way we did when " +
           "we looked at the fact that bk\u00a0=\u00a0vr. There are now " +
                           "(3\u00a0\u2212\u00a01) 1s in each of the 6 columns",SET_MARK);
    sleep(delay*3);

    int total = 0;
    int counter = 0;
    for(int j = 0; j < m.getColumns(); j++) {
      counter = 0;
      m.colorColumnLabel(0,j,Color.RED);
      m.colorColumnLabel(1,j,Color.RED);
      for(int i = 0; i < m.getRows(); i++) {
        if(m.getEntry(i,j) == 1) {
          m.colorEntry(i,j,Color.RED);
          m.setColumnLabel(1,j,++counter);
          m.setCornerLabel(++total);
          sleep(delay);
        }
      }
      m.colorColumn(j,Color.BLACK);       
      m.colorColumnLabel(0,j,Color.BLACK);
      m.colorColumnLabel(1,j,Color.BLACK);
    }
    details.append(", so");
        details.append(MathSymbol.getTextSymbol("r(k \u2212 1) = 6 \u00d7 2 = 12").toMathExpression(16f),SCROLL);

      }
    }

    /**
     * Sums the number of 1s in the rows (lambda(v-1))
     */
    private class Part8 implements AnimationPart {
      public void animate() {
    for(int i = 0; i < m.getColumns(); i++)
      m.setColumnLabel(1,i,"");
    m.setCornerLabel("");
      
    details.append("There are now 2 1s in each of the (7\u00a0\u2212\u00a01) rows",SET_MARK|SCROLL);
    sleep(delay*3);

    int total = 0;
    int counter = 0;
    for(int i = 0; i < m.getRows(); i++) {
      counter = 0;
      m.colorRowLabel(0,i,Color.BLUE);
      m.colorRowLabel(1,i,Color.BLUE);
      for(int j = 0; j < m.getColumns(); j++) {
        if(m.getEntry(i,j) == 1) {
          m.colorEntry(i,j,Color.BLUE);
          m.setRowLabel(1,i,++counter);
          m.setCornerLabel(++total);
          sleep(delay);
        }
      }
      m.colorRow(i,Color.BLACK);
      m.colorRowLabel(0,i,Color.BLACK);
      m.colorRowLabel(1,i,Color.BLACK);
    }
    details.append(", so");
        details.append(MathSymbol.getTextSymbol("\u03bb(v \u2212 1) = 2 \u00d7 6 = 12").toMathExpression(16f),SCROLL);
      }
    }

    private class Part9 implements AnimationPart {
      public void animate() {
    details.append("It is easy to see that the second part of the theorem holds in this example.",SET_MARK|SCROLL);
    for(int i = 0; i < m.getRows(); i++)
      m.setRowLabel(1,i,"");
    m.setCornerLabel("");  
      }
    }

  }


  /**
   * Returns the unicode character subscript for the given integer.
   *
   * @param n an integer
   * @return the unicode subscript for n
   */
  private String getSubscript(int n) {
    String str = n + "";
    StringBuffer sb = new StringBuffer();

    for(int i = 0; i < str.length(); i++)
      sb.append((char)(Integer.parseInt(str.substring(i,i+1))+8320));

    return sb.toString();
  }

}
