package modules.fibonacci;

import java.awt.*;
import java.awt.geom.*;

public class Board {
  public final static int SQUARE = 0;
  public final static int DOMINO1 = 1;
  public final static int DOMINO2 = 2;

  public final static Color OUTLINE_NORMAL = Color.BLACK;
  public final static Color OUTLINE_HIGHLIGHT = Color.ORANGE;

  public final static Color SQUARE_COLOR_NORMAL = Color.BLUE.darker().darker();
  public final static Color DOMINOE_COLOR_NORMAL = Color.RED.darker().darker();
  public final static Color SQUARE_COLOR_HIGHLIGHT = 
    Color.BLUE.brighter().brighter().brighter();
  public final static Color DOMINOE_COLOR_HIGHLIGHT = 
    Color.RED.brighter().brighter().brighter();
  
  Color squareColor = SQUARE_COLOR_NORMAL;
  Color dominoeColor = DOMINOE_COLOR_NORMAL;
  
  Color[] outlineColor;

  boolean boardHighlight = false;
  boolean[] highlight;

  boolean[] breakHighlight;
  Color breakHighlightColor = OUTLINE_HIGHLIGHT;

  int n;
  float size;

  public float x;
  public float y;

  float alpha;

  int[] tiling;

  public Board(int[] t, float sz) {
    this(t.length,sz);

    tiling = t;
  }

  public Board(int num, float sz) {
    n = num;
    size = sz;

    tiling = new int[n];
    highlight = new boolean[n];

    outlineColor = new Color[n];

    breakHighlight = new boolean[n+1];

    for(int i = 0; i < outlineColor.length; i++)
      outlineColor[i] = OUTLINE_NORMAL;
    
    alpha = 1f;
  }

  public void setAlpha(float a) {
    float[] f;
    alpha = a;
    f = squareColor.getColorComponents(null);
    squareColor = new Color(f[0],f[1],f[2],a);
    f = dominoeColor.getColorComponents(null);
    dominoeColor = new Color(f[0],f[1],f[2],a);

    for(int i = 0; i < outlineColor.length; i++) {
      f = outlineColor[i].getColorComponents(null);
      outlineColor[i] = new Color(f[0],f[1],f[2],a);      
    }

    f = OUTLINE_HIGHLIGHT.getColorComponents(null);
    breakHighlightColor = new Color(f[0],f[1],f[2],alpha);
  }

  public void setDomino(int i) {
    if(i > tiling.length - 2)
      throw new IllegalArgumentException();

    tiling[i] = DOMINO1;
    tiling[i+1] = DOMINO2;
  }

  public void setDominoes(int[] index) {
    for(int i = 0; i < index.length; i++)
      setDomino(index[i]);
  }

  public void setTiling(int[] t) {
    tiling = t;
    n = tiling.length;
    outlineColor = new Color[n];
    breakHighlight = new boolean[n+1];

    for(int i = 0; i < outlineColor.length; i++)
      outlineColor[i] = OUTLINE_NORMAL;
  }

  public void setSize(float sz) {
    size = sz;
  }

  public void highlight(int i, boolean v) {
    highlight[i] = v;

    float[] f;

    if(highlight[i]) {
      f = OUTLINE_HIGHLIGHT.getColorComponents(null);
      outlineColor[i] = new Color(f[0],f[1],f[2],alpha);
    } else {
      f = OUTLINE_NORMAL.getColorComponents(null);
      outlineColor[i] = new Color(f[0],f[1],f[2],alpha);
    }
  }

  public void highlight(boolean v) {
    float[] f;

    boardHighlight = v;
    if(boardHighlight) {
      f = SQUARE_COLOR_HIGHLIGHT.getColorComponents(null);
      squareColor = new Color(f[0],f[1],f[2],alpha);
      f = DOMINOE_COLOR_HIGHLIGHT.getColorComponents(null);
      dominoeColor = new Color(f[0],f[1],f[2],alpha);
    } else {
      f = SQUARE_COLOR_NORMAL.getColorComponents(null);
      squareColor = new Color(f[0],f[1],f[2],alpha);
      f = DOMINOE_COLOR_NORMAL.getColorComponents(null);
      dominoeColor = new Color(f[0],f[1],f[2],alpha);
    }
  }

  public void breakHighlight(int i, boolean v) {
    breakHighlight[i] = v;
  }

  public float getWidth() {
    return n * size;
  }

  public float getHeight() {
    return size;
  }

  public void draw(Graphics2D g) {
    Rectangle2D.Float r = null;
    Rectangle2D.Float rh = null;
    Color rhcolor = null;
    float[] f;
    
    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                       RenderingHints.VALUE_ANTIALIAS_ON);

    BasicStroke s = new BasicStroke(3);
    BasicStroke dashed = new BasicStroke(3,BasicStroke.CAP_BUTT,
                                         BasicStroke.JOIN_MITER,
                                         10f,
                                         new float[] {2f, 4f},
                                         0f);
    int i;
    for(i = 0; i < n; i++) {
      g.setStroke(s);
      if(i > 0 && highlight[i-1]) {
        rh = r;
        rhcolor = outlineColor[i-1];
      } else if(i > 0 && tiling[i-1] != DOMINO2) {
        rh = null;
        rhcolor = null;
      }
      if(tiling[i] == SQUARE) {
        r = new Rectangle2D.Float
          (x+size*i,y,size,size);
        g.setColor(squareColor);
        g.fill(r);
        g.setColor(outlineColor[i]);
        g.draw(r);
      } else if(tiling[i] == DOMINO1) {
        r = new Rectangle2D.Float
          (x+size*i,y,size*2,size);
        g.setColor(dominoeColor);
        g.fill(r);
        g.setColor(outlineColor[i]);
        g.draw(r);
      }
      if(rh != null) {
        g.setColor(rhcolor);
        g.draw(rh);        
      }
      if(breakHighlight[i]) {
        if(i > 0 &&  tiling[i-1] == DOMINO1) {
          g.setColor(breakHighlightColor);
          //g.setColor(Color.BLACK);
          g.setStroke(dashed);
          g.draw(new Line2D.Float(x+size*i, y, x+size*i, y+size));
        } else {
          g.setColor(breakHighlightColor);
          //g.setColor(Color.BLACK);
          g.setStroke(s);
          g.draw(new Line2D.Float(x+size*i, y, x+size*i, y+size));
        }
      }
    }
    if(breakHighlight[i]) {
      if(i > 0 &&  tiling[i-1] == DOMINO1) {
        g.setColor(breakHighlightColor);
        //g.setColor(Color.BLACK);
        g.setStroke(dashed);
        g.draw(new Line2D.Float(x+size*i, y, x+size*i, y+size));
      } else {
        g.setColor(breakHighlightColor);
        //g.setColor(Color.BLACK);
        g.setStroke(s);
        g.draw(new Line2D.Float(x+size*i, y, x+size*i, y+size));
      }
    }

  }
}