package modules.fibonacci2;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

import modules.fibonacci.Board;

import vcp.animation.*;
import vcp.mathnotation.*;
import vcp.utilities.ScrollableTextArea;


import info.clearthought.layout.TableLayout;

public class Fibonacci2 extends JPanel 
  implements 
    vcp.utilities.ScrollableTextAreaConstants {

  ControlPanel cp;

  AnimationCanvas canvas;

  ScrollableTextArea details;

  public Fibonacci2() {
    super(new TableLayout
          (new double[][]
            {{TableLayout.FILL},
             {TableLayout.PREFERRED,
              TableLayout.PREFERRED,
              TableLayout.FILL,
              TableLayout.PREFERRED}}));

    cp = new SliderControlPanel();

    canvas = new AnimationCanvas(335,395,getBackground());
    
    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));
    details.setMargins(6);
 
    MathExpression expr = new MathExpression
      (new MathSymbol[]
        {MathSymbol.getSummationSymbol
         ("k=0","n",
          MathSymbol.getSubscript("f","k"),
          "sum of f_k with k = 0 to n"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getSubscript("f","n+2"),
         MathSymbol.getTextSymbol("\u2212 1")
        },24f);
      
    JLabel theorem = new JLabel
      (new ImageIcon(expr.toImage(10)));

    add(theorem,"0,0,c,c");
    add(new JScrollPane(canvas),"0,1,c,c");
    add(details.getJScrollPane(),"0,2");
    add(cp,"0,3");
  }

  public void addNotify() {
    super.addNotify();

    new Fib2Animation();
  }

  private class Fib2Animation extends FPSAnimation {
    static final float BOARD_SIZE = 25;
    static final int WAIT = 30;
    static final float LEFT_SIDE = 5;
    static final float RIGHT_SIDE = 180;
    static final float BASE = 5 + (BOARD_SIZE+5)*12;
    
    Board[] boards;

    public Fib2Animation() {
      super(cp,60);

      setPartArray(createPartArray());

      boards = new Board[13];

      float y = 5;
      for(int i = 0; i < boards.length; i++) {
        boards[i] = new Board(6,BOARD_SIZE);
        boards[i].x = RIGHT_SIDE;
        boards[i].y = y;
        y += BOARD_SIZE + 5;
      }

      for(int i = 1; i < 6; i++)
        boards[i].setDomino(i-1);

      boards[6].setDominoes(new int[] {0,2});
      boards[7].setDominoes(new int[] {0,3});
      boards[8].setDominoes(new int[] {0,4});

      boards[9].setDominoes(new int[] {1,3});
      boards[10].setDominoes(new int[] {1,4});

      boards[11].setDominoes(new int[] {2,4});
      boards[12].setDominoes(new int[] {0,2,4});

      drawBoards();
    }

    public AnimationPart[] createPartArray() {
      return new AnimationPart[]
        {
          new Part0(),
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5(),
          new Part6(),
          new Part7(),
          new Part8()
        };
    }

    private void drawBoards() {
      Graphics2D g = (Graphics2D) canvas.getImageGraphics();

      drawBoards(g);

      g.dispose();

      canvas.paintCanvas();
    }

    private void drawBoards(Graphics2D g) {
      for(int i = 0; i < boards.length; i++)
        if(boards[i] != null)
          boards[i].draw(g);      
    }

    private boolean moveBoards() {
      boolean done = true;
      Board leftTemp[] = new Board[boards.length];
      Board left[] = new Board[boards.length];
      Board right[] = new Board[boards.length];
      int ri = 0;
      int li = 0;
      
      for(int i = boards.length - 1; i >= 0; i--) {
        if(boards[i] != null)
          if(boards[i].x == RIGHT_SIDE)
            right[ri++] = boards[i];
      }

      for(int i = 0; i < boards.length; i++) {
        if(boards[i] != null)
          if(boards[i].x == LEFT_SIDE) {
            leftTemp[(int)(boards[i].y/(BOARD_SIZE+5))] = 
              boards[i];
          }
      }

      for(int i = 0; i < boards.length; i++)
        if(leftTemp[i] != null)
          left[li++] = leftTemp[i];

      for(int i = 0; i < right.length; i++) {
        if(right[i] != null) {
          if(right[i].y < (BASE - (BOARD_SIZE+5)*i)) {
            done = false;
            right[i].y += 2; 
          }
          if(right[i].y > (BASE - (BOARD_SIZE+5)*i))
            right[i].y = (BASE - (BOARD_SIZE+5)*i);
        }
        if(left[i] != null) {
          if(left[i].y  > (35 + (BOARD_SIZE+5)*i)) {
            done = false;
            left[i].y -= 2;
          }
          if(left[i].y < (35 + (BOARD_SIZE+5)*i))
            left[i].y = (35 + (BOARD_SIZE+5)*i);            
        }
      }

      return done;
    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.setContent("We start with all the tilings for a board of length 4+2 = 6. There are f\u2086 = 13 ways to do this.");
      }
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        details.append("If we remove the \"all square\" tiling, the number of tilings left is",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[] 
             {MathSymbol.getSubscript("f","4+2"),
              MathSymbol.getTextSymbol("\u2212 1"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("12")},20f));
        details.append("This is the right-hand side of the identity.",SCROLL);

        for(float alpha = 1; alpha >= 0; alpha -= .1) {
          boards[0].setAlpha(alpha);
          drawBoards();
          sleep();
        }
        boards[0] = null;
        drawBoards();
      }
    }

    private class Part2 implements AnimationPart {
      public void animate() {
        details.append("Now we will count the left-hand side of the identity by counting tilings based on the location of the last domino.",NEWLINES|SET_MARK|SCROLL);
      }
    }

    private class Part3 implements AnimationPart {
      public void animate() {
        details.append("There is",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[] 
             {MathSymbol.getSubscript("f","0"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("1")},17f));
        details.append("way to tile a 6-board with the last domino covering cells 0+1 and 0+2.");
        details.scrollToMark();
       

        boards[1].highlight(true);
        boards[1].highlight(0,true);
        drawBoards();

        sleep(WAIT);

        float xo = boards[1].x;
        for(float x = xo; x >= LEFT_SIDE; x-=2) {
          boards[1].x = x;
          drawBoards();
          sleep();
        }
        boards[1].x = LEFT_SIDE;
        drawBoards();
      }
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        details.append("There is",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[] 
             {MathSymbol.getSubscript("f","1"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("1")},17f));
        details.append("way to tile a 6-board with the last domino covering cells 1+1 and 1+2.");
        details.scrollToMark();

        boards[2].highlight(true);
        boards[2].highlight(1,true);

        boards[1].highlight(false);
        boards[1].highlight(0,false);
        drawBoards();

        sleep(WAIT);

        float xo = boards[2].x;
        for(float x = xo; x >= LEFT_SIDE; x-=2) {
          boards[2].x = x;
          drawBoards();
          sleep();
        }
        boards[2].x = LEFT_SIDE;
        drawBoards();
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        details.append("There are",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[] 
             {MathSymbol.getSubscript("f","2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("2")},17f));
        details.append("ways to tile a 6-board with the last domino covering cells 2+1 and 2+2.");
        details.scrollToMark();

        boards[3].highlight(true);
        boards[3].highlight(2,true);        
        boards[6].highlight(true);
        boards[6].highlight(2,true);        

        boards[2].highlight(false);
        boards[2].highlight(1,false);
        drawBoards();

        sleep(WAIT);

        float xo = boards[3].x;
        for(float x = xo; x >= LEFT_SIDE; x-=2) {
          boards[3].x = x;
          boards[6].x = x;
          drawBoards();
          sleep();
        }
        boards[3].x = LEFT_SIDE;
        boards[6].x = LEFT_SIDE;
        drawBoards();
        
        while(!moveBoards()) {
          drawBoards();
          sleep();
        }
      }
    }

    private class Part6 implements AnimationPart {
      public void animate() {
        details.append("There are",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[] 
             {MathSymbol.getSubscript("f","3"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("3")},17f));
        details.append("ways to tile a 6-board with the last domino covering cells 3+1 and 3+2.");
        details.scrollToMark();

        boards[3].highlight(false);
        boards[3].highlight(2,false);        
        boards[6].highlight(false);
        boards[6].highlight(2,false); 

        boards[4].highlight(true);
        boards[4].highlight(3,true);
        boards[7].highlight(true);
        boards[7].highlight(3,true);
        boards[9].highlight(true);
        boards[9].highlight(3,true);
        drawBoards();

        sleep(WAIT);

        float xo = boards[4].x;
        for(float x = xo; x >= LEFT_SIDE; x-=2) {
          boards[4].x = x;
          boards[7].x = x;
          boards[9].x = x;
          drawBoards();
          sleep();
        }
        boards[4].x = LEFT_SIDE;
        boards[7].x = LEFT_SIDE;
        boards[9].x = LEFT_SIDE;
        drawBoards();

        while(!moveBoards()) {
          drawBoards();
          sleep();
        }
      }
    }

    private class Part7 implements AnimationPart {
      public void animate() {
        details.append("\nLastly, there are",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[] 
             {MathSymbol.getSubscript("f","4"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("5")},17f));
        details.append("ways to tile a 6-board with the last domino covering cells 4+1 and 4+2.");
        details.scrollToMark();

        boards[5].highlight(true);
        boards[5].highlight(4,true);
        boards[8].highlight(true);
        boards[8].highlight(4,true);
        boards[10].highlight(true);
        boards[10].highlight(4,true);        
        boards[11].highlight(true);
        boards[11].highlight(4,true);        
        boards[12].highlight(true);
        boards[12].highlight(4,true);        

        boards[4].highlight(false);
        boards[4].highlight(3,false);
        boards[7].highlight(false);
        boards[7].highlight(3,false);
        boards[9].highlight(false);
        boards[9].highlight(3,false);
        drawBoards();

        sleep(WAIT);

        float xo = boards[5].x;
        for(float x = xo; x >= LEFT_SIDE; x-=2) {
          boards[5].x = x;
          boards[8].x = x;
          boards[10].x = x;
          boards[11].x = x;
          boards[12].x = x;
          drawBoards();
          sleep();
        }

        boards[5].x = LEFT_SIDE;
        boards[8].x = LEFT_SIDE;
        boards[10].x = LEFT_SIDE;
        boards[11].x = LEFT_SIDE;
        boards[12].x = LEFT_SIDE;
        drawBoards();

        while(!moveBoards()) {
          drawBoards();
          sleep();
        }
      }
    }

    private class Part8 implements AnimationPart {
      public void animate() {
        details.append("It is clear that the theorem holds in this example,",
                       NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSummationSymbol
              ("k=0","4",
               MathSymbol.getSubscript("f","k"),
               "sum of f_k with k = 0 to 4"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("12"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSubscript("f","4+2"),
              MathSymbol.getTextSymbol("\u2212 1")
             },20f));
        details.scrollToMark();

        boards[5].highlight(false);
        boards[5].highlight(4,false);
        boards[8].highlight(false);
        boards[8].highlight(4,false);
        boards[10].highlight(false);
        boards[10].highlight(4,false);        
        boards[11].highlight(false);
        boards[11].highlight(4,false);        
        boards[12].highlight(false);
        boards[12].highlight(4,false);        

        drawBoards();
      }
    }
  }
}
