package modules.fibonacci4;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;

import modules.fibonacci.Board;

import vcp.animation.*;
import vcp.mathnotation.*;
import vcp.utilities.ScrollableTextArea;

import info.clearthought.layout.TableLayout;

public class Fibonacci4 extends JPanel 
  implements 
    vcp.utilities.ScrollableTextAreaConstants {

  ControlPanel cp;

  AnimationCanvas canvas;

  ScrollableTextArea details;

  public Fibonacci4() {
    super(new TableLayout
          (new double[][]
            {{TableLayout.FILL},
             {TableLayout.PREFERRED,
              TableLayout.PREFERRED,
              TableLayout.FILL,
              TableLayout.PREFERRED}}));

    cp = new SliderControlPanel();

    canvas = new AnimationCanvas(241,355,getBackground());
    
    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));
    details.setMargins(6);
 
    MathExpression expr = new MathExpression
      (new MathSymbol[]
        {MathSymbol.getSummationSymbol
         ("k=0","n",
          MathSymbol.getSuperscript
          (MathSymbol.getSubscript("f","k").toMathExpression(),
           MathSymbol.getTextSymbol("2").toMathExpression()),
          "sum of f_k^2 with k = 0 to n"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getSubscript("f","n"),
         MathSpace.getNoSpace(),
         MathSymbol.getSubscript("f","n+1")},30f);

    JLabel theorem = new JLabel
      (new ImageIcon(expr.toImage(10)));

    add(theorem,"0,0,c,c");
    add(new JScrollPane(canvas),"0,1,c,c");
    add(details.getJScrollPane(),"0,2");
    add(cp,"0,3");
  }

  public void addNotify() {
    super.addNotify();

    new Fib4Animation();
  }  

  protected class Fib4Animation extends FPSAnimation {
    static final float BOARD_SIZE = 20;
    static final float LEFT_SIDE = 5;
    static final float RIGHT_SIDE = 160;

    Board[] boards3;
    Board[] boards2;

    public Fib4Animation() {
      super(cp, 60);

      setPartArray(createPartArray());

      boards3 = new Board[6];
      boards2 = new Board[6];

      for(int i = 0; i < 6; i++) {
        boards3[i] = new Board(3,BOARD_SIZE);
        boards3[i].x = RIGHT_SIDE; 
        boards3[i].y = 5 + (BOARD_SIZE+10) * i * 2;
        boards2[i] = new Board(2,BOARD_SIZE);
        boards2[i].x = RIGHT_SIDE;
        boards2[i].y = 10 + BOARD_SIZE + (BOARD_SIZE+10) * i * 2;
        boards2[i].setAlpha(0);
      }
      /*
      for(int i = 4; i < 6; i++) {
        boards3[i] = new Board(3,BOARD_SIZE);
        boards3[i].x = RIGHT_SIDE; 
        boards3[i].y = 5 + (BOARD_SIZE+10) * (i-4) * 2;
        boards2[i] = new Board(2,BOARD_SIZE);
        boards2[i].x = RIGHT_SIDE;
        boards2[i].y = 10 + BOARD_SIZE + (BOARD_SIZE+10) * (i-4) * 2;
      }
      */
      boards2[1].setDomino(0);
      boards2[3].setDomino(0);
      boards2[5].setDomino(0);

      boards3[2].setDomino(0);
      boards3[3].setDomino(0);

      boards3[4].setDomino(1);
      boards3[5].setDomino(1);
      
      boards3[1].setAlpha(0);
      boards3[3].setAlpha(0);
      boards3[5].setAlpha(0);



      drawBoards();
    }

    public void drawBoards() {
      Graphics2D g = (Graphics2D) canvas.getImageGraphics();
      drawBoards(g);
      canvas.paintCanvas();
      g.dispose();
    }

    public void drawBoards(Graphics2D g) {
      for(int i = 0; i < boards3.length; i++)
        boards3[i].draw(g);

      for(int i = 0; i < boards2.length; i++)
        boards2[i].draw(g);
    }

    public AnimationPart[] createPartArray() {
      return new AnimationPart[]
        {
          new Part0(),
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5(),
          new Part6(),
        };
    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.setContent("We start with all the (n+1)-tilings for n = 2,");
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSubscript("f","2+1"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("3")
             },20f));
        details.append("We want all the pairs of tilings with one 2-board and one (2+1)-board. These are independent so we multiply them. Since there are f\u2082 = 2 tilings of a 2-board, we need 2 of each of the (2+1)-tilings.");
      }
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        for(float alpha = 0; alpha <= 1; alpha += .05) {
          boards3[1].setAlpha(alpha);
          boards3[3].setAlpha(alpha);
          boards3[5].setAlpha(alpha); 
          drawBoards();
          sleep();
        }
        boards3[1].setAlpha(1);
        boards3[3].setAlpha(1);
        boards3[5].setAlpha(1); 
        drawBoards();
        sleep(20);

        for(float alpha = 0; alpha <= 1; alpha += .05) {
          for(int i = 0; i < boards2.length; i++)
            boards2[i].setAlpha(alpha); 
          drawBoards();
          sleep();
        }
        for(int i = 0; i < boards2.length; i++)
          boards2[i].setAlpha(1); 
        drawBoards();
        sleep();

        details.append("We have now counted the right-hand side of the identity for n = 2,",
                       NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSubscript("f","2"),
              MathSpace.getNoSpace(),
              MathSymbol.getSubscript("f","2+1"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("6")
             },20f));
        details.append("Make sure to note that throughout, there is always only one way to tile the tail; all of the following results are assumed to be multiplied by one.",SCROLL);      
      }
    }

    private class Part2 implements AnimationPart {
      public void animate() {
        details.append("Now, we will count the left-hand side of the identity by counting the number of tilings based on the position of the last fault. To help visualize this, we highlight the last fault in each of the board pairs.",NEWLINES|SET_MARK|SCROLL);

        boards3[5].breakHighlight(0,true);        
        boards2[5].breakHighlight(0,true);        
        boards3[4].breakHighlight(1,true);        
        boards2[4].breakHighlight(1,true);        
        for(int i = 0; i < 4; i++) {
          boards3[i].breakHighlight(2,true);
          boards2[i].breakHighlight(2,true);
        }
        drawBoards();
      }
    }

    private class Part3 implements AnimationPart {
      public void animate() {
        details.append("The number of ways to tile a 2-board and a 3-board with the last fault at k = 0 is",NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSuperscript
              (MathSymbol.getSubscript("f","0").toMathExpression(),
               MathSymbol.getTextSymbol("2").toMathExpression()),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSuperscript("1","2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("1")}, 20f),SCROLL);

        boards3[5].highlight(true);
        boards2[5].highlight(true);
        drawBoards();

        sleep(30);

        for(float x = boards3[5].x; x >= LEFT_SIDE; x -= 2) {
          boards3[5].x = x;
          boards2[5].x = x;

          drawBoards();
          sleep();
        }
        boards3[5].x = LEFT_SIDE;
        boards2[5].x = LEFT_SIDE;
        drawBoards();

        
      }
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        details.append("The number of ways to tile a 2-board and a  3-board with the last fault at k = 1 is",SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSuperscript
              (MathSymbol.getSubscript("f","1").toMathExpression(),
               MathSymbol.getTextSymbol("2").toMathExpression()),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSuperscript("1","2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("1")}, 20f),SCROLL);

        boards3[5].highlight(false);
        boards2[5].highlight(false);

        boards3[4].highlight(true);
        boards2[4].highlight(true);
        drawBoards();

        sleep(30);

        for(float x = boards3[4].x; x >= LEFT_SIDE; x -= 2) {
          boards3[4].x = x;
          boards2[4].x = x;

          drawBoards();
          sleep();
        }
        boards3[4].x = LEFT_SIDE;
        boards2[4].x = LEFT_SIDE;
        drawBoards();
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        details.append("The number of ways to tile a 2-board and a 3-board with the last fault at k = n = 2 is",SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSuperscript
              (MathSymbol.getSubscript("f","2").toMathExpression(),
               MathSymbol.getTextSymbol("2").toMathExpression()),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSuperscript("2","2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("4")}, 20f),SCROLL);

        boards3[4].highlight(false);
        boards2[4].highlight(false);

        for(int i = 0; i < 4; i++) {
          boards3[i].highlight(true);
          boards2[i].highlight(true);
        }
        
        drawBoards();

        sleep(30);

        for(float x = boards3[0].x; x >= LEFT_SIDE; x -= 2) {
          for(int i = 0; i < 4; i++) {
            boards3[i].x = x;
            boards2[i].x = x;
          }

          drawBoards();
          sleep();
        }
        for(int i = 0; i < 4; i++) {
          boards3[i].x = LEFT_SIDE;
          boards2[i].x = LEFT_SIDE;
        }
        drawBoards();
      }
    }

    private class Part6 implements AnimationPart {
      public void animate() {
        for(int i = 0; i < 4; i++) {
          boards3[i].highlight(false);
          boards2[i].highlight(false);
        }        

        drawBoards();

        details.append("It is clear that the theorem holds in this example,",
                       SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSummationSymbol
         ("k=0","2",
          MathSymbol.getSuperscript
          (MathSymbol.getSubscript("f","k").toMathExpression(),
           MathSymbol.getTextSymbol("2").toMathExpression()),
          "sum of f_k^2 with k = 0 to 2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("6"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSubscript("f","2"),
              MathSpace.getNoSpace(),
              MathSymbol.getSubscript("f","2+1")
             },20f),SCROLL);
      }
    }
  }
}
