package modules.oresme;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.font.*;

import vcp.utilities.*;

public class UnitPart {
  float speed = 5;

  Font font;

  float w = 250;
  float h = 50;

  float x, y;

  float part;
  float labelAlpha = 1f;

  Color color;

  Rectangle2D.Float rect;
  GlyphVector label;
  String labelText;
  Stroke stroke;
  float labelWidth;
      
  public UnitPart(String l, float p) {
    this(l,p,true);
  }

  public UnitPart(String l, float p, boolean ex) {
    part = p;
        
    if(ex) 
      rect = new Rectangle2D.Float(0,0,w/part,h);
    else 
      rect = new Rectangle2D.Float(0,0,h,h);

    stroke = new BasicStroke(3);

    labelText = l;
    //labelText = null;

    font = null;
    labelWidth = 0;

    color = Color.BLACK;
  }

  public boolean expand() {
    if(rect.width + speed <= w/part)
      rect.width += speed;
    else
      rect.width = w/part;

    return rect.width == w/part;
  }

  public void setAlpha(float a) {
    labelAlpha = a;
    float[] f = color.getColorComponents(null);
    color = new Color(f[0],f[1],f[2],a);
  }

  public void setLabelAlpha(float a) {
    labelAlpha = a;
  }

  public float getWidth() {
    return rect.width;
  }

  public float getHeight() {
    return rect.height;
  }

  private void drawDivisions(Graphics2D g2d) {
    //needs work
    double xval = x;
    double d = rect.width;
    for(int i = 0; i < 5; i++) {
      d /= 2.0;
      xval += d;//Math.ceil(xval+d);
      g2d.draw(new Line2D.Double(xval,(double)y,xval,(double)(y+rect.height)));
    }
  }

  public boolean move(float xa, float ya) {
    
    if(x+speed <= xa) 
      x+=speed;
    else if (x-speed >= xa)
      x-=speed;
    else
      x = xa;
      

    if(y+speed <= ya) 
      y+=speed;
    else if (y-speed >= ya)
      y-=speed;
    else
      y = ya;


    return (x == xa) && (y == ya);
  }

  public void setLabel(String l) {
    labelText = l;
  }

  public boolean fadeLabel(String l, float d) {
    labelAlpha -= d;
    if(labelAlpha < 0)
      labelAlpha = 0;

    return labelAlpha == 0;
  }

  public void draw(Graphics2D g, float xa, float ya) {
    x = xa;
    y = ya;

    draw(g);
  }

  private Font getFont() {
    try {
      return UnicodeUtil.getGreekFont(UnicodeUtil.SANS).deriveFont(Font.BOLD);
    } catch (NoFontFoundException e) {
      for(int i = 0; i < e.getFontList().length; i++) {
        System.out.println(e.getFontList()[i]);
      }
      return null;
    } 
  }

  public void draw(Graphics2D ga) {
    Graphics2D g = (Graphics2D) ga.create();

    rect.x = x;
    rect.y = y;

    g.setStroke(stroke);
    g.setColor(color);
    //drawDivisions(g);
    g.draw(rect);

    if (labelText != null) {
      if(font == null) {
        font = getFont().deriveFont(12f);
        FontMetrics fm = g.getFontMetrics(font);
        float fh = fm.getAscent() + fm.getDescent();
        font = font.deriveFont((12f*20f)/fh);
        labelWidth = (float) fm.stringWidth(labelText);
      }
      
      label = font.createGlyphVector
        (g.getFontRenderContext(),labelText);

      float[] f = color.getColorComponents(null);
      Color lcolor = new Color(f[0],f[1],f[2],labelAlpha);      

      g.setColor(lcolor);
      g.drawGlyphVector(label, x + rect.width/2f - labelWidth/2f, y+35);
    }
  }
}