package modules.pascal;

import javax.swing.*;
import java.awt.*;

import vcp.utilities.ScrollableTextArea;
import vcp.utilities.ExtendedTableLayout;
import vcp.mathnotation.*;
import vcp.animation.*;

import info.clearthought.layout.TableLayout;

public class PascalVisualization extends JPanel 
  implements
    vcp.utilities.ScrollableTextAreaConstants {

  PlayerTable playerTable;
  PascalTriangle pascalTriangle;

  ScrollableTextArea details;
  
  ControlPanel cp;

  JLabel total;
  JLabel noStart;
  JLabel start;
  JLabel sum;

  JPanel left;

  public PascalVisualization() {
    super(new TableLayout(new double[][]
      {{TableLayout.FILL,
        TableLayout.FILL},
       {TableLayout.PREFERRED,
        TableLayout.FILL,
        TableLayout.PREFERRED}}));

    TableLayout rightTL;

    JPanel right = new JPanel
      (rightTL = new TableLayout
       (new double[][]
         {{TableLayout.PREFERRED},
          {TableLayout.PREFERRED,
           TableLayout.PREFERRED,
           TableLayout.PREFERRED,
           TableLayout.PREFERRED,}}));

    rightTL.setVGap(10);

    MathExpression expr = new MathExpression
      (new MathSymbol[] 
        {MathSymbol.getBinomialCoefficient("7","5"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getTextSymbol("21")},25f);
    expr.setColor(Color.GREEN.darker().darker());

    total = new JLabel(new ImageIcon(expr.toImage(10)));
    total.setVisible(false);

    expr = new MathExpression
      (new MathSymbol[] 
        {MathSymbol.getBinomialCoefficient("7\u22121","5"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getBinomialCoefficient("6","5"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getTextSymbol("6")},25f);
    expr.setColor(Color.RED);

    noStart = new JLabel(new ImageIcon(expr.toImage(10)));
    noStart.setVisible(false);

    expr = new MathExpression
      (new MathSymbol[] 
        {MathSymbol.getBinomialCoefficient("7\u22121","5-1"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getBinomialCoefficient("6","4"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getTextSymbol("15")},25f);
    expr.setColor(Color.BLUE);

    start = new JLabel(new ImageIcon(expr.toImage(10)));
    start.setVisible(false);

    MathSymbol six = MathSymbol.getTextSymbol("6");
    MathSymbol fifteen = MathSymbol.getTextSymbol("15");
    MathSymbol twentyone = MathSymbol.getTextSymbol("21");
  
    
    expr = new MathExpression
      (new MathSymbol[]
        {six,
         MathSymbol.getTextSymbol("+"),
         fifteen,
         MathSymbol.getTextSymbol("="),
         twentyone},25f);
    expr.setColor(Color.BLACK);
    six.setColor(Color.RED);
    fifteen.setColor(Color.BLUE);
    twentyone.setColor(Color.GREEN.darker().darker());

    sum = new JLabel(new ImageIcon(expr.toImage(10)));
    sum.setVisible(false);

    right.add(total,"0,0,r,c");
    right.add(noStart,"0,1,r,c");
    right.add(start,"0,2,r,c");
    right.add(sum,"0,3,r,c");

    cp = new SliderControlPanel();
    new PascalAnimation();

    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));
    details.setMargins(6);

    details.setContent("Consider a basketball team of 7 players. The players are MJ, BR, KAJ, LB, PM, DW, and OR. How many ways can the coach choose five players for the starting line-up?");

    pascalTriangle = new PascalTriangle();

    expr = new MathExpression
      (new MathSymbol[]
        {MathSymbol.getBinomialCoefficient("n","r"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getBinomialCoefficient("n\u22121","r"),
         MathSymbol.getTextSymbol("+"),
         MathSymbol.getBinomialCoefficient("n\u22121","r-1")},30f);
    
    JLabel identity  = 
      new JLabel("Pascal's Identity: ",
                 new ImageIcon(expr.toImage(10)),
                 JLabel.CENTER);

    identity.setHorizontalTextPosition(JLabel.LEFT);

      
    left = new JPanel(new TableLayout
                      (new double[][]
                        {{TableLayout.FILL},
                         {TableLayout.FILL}}));
    //left.setBorder(new javax.swing.border.LineBorder(Color.black,1));
    left.add(playerTable = new PlayerTable(),"0,0,C,C");

    add(left,"0,0,c,c");
    add(right,"1,0,c,c");
    add(identity,"1,0,r,b");
    add(details.getJScrollPane(),"0,1,1,1");
    add(cp,"0,2,1,2");

    left.validate();
    //left.setMinimumSize(left.getPreferredSize());
    left.setPreferredSize(new java.awt.Dimension
                          ((int)pascalTriangle.getPreferredSize().getWidth(),
                           (int)left.getPreferredSize().getHeight()));

    playerTable.setVisible(false);
  }

  private class PascalTriangle extends JPanel {
    JLabel[][] entries;

    public PascalTriangle() {
      super();
      
      //setBorder(new javax.swing.border.LineBorder(Color.green,1));

      double[][] table = new double[2][];
      
      //for(int i = 0; i < table.length; i++) {
      
      table[0] = new double[15];
      table[1] = new double[8];
      for(int j = 0; j < table[0].length; j++)
        table[0][j] = 20;
      for(int j = 0; j < table[1].length; j++)
        table[1][j] = 20;
      
      setLayout(new TableLayout(table));
      
      entries = new JLabel[8][];
      
      for(int i = 0; i < entries.length; i++) {
        entries[i] = new JLabel[i+1];
        for(int j = 0; j < entries[i].length; j++) {
          entries[i][j] = new JLabel(comb(i,j)+"");
          add(entries[i][j],(7-i+(j*2))+","+i+",c,c");
        }
        
      }
    }

    public void colorEntry(int i, int j, Color c) {
      entries[i][j].setForeground(c);
    }
    
    private int comb(int n, int r) {
      return fact(n)/(fact(r)*fact(n-r));
    }

    private int fact(int f) {
      if(f == 0)
        return 1;

      int r = 1;

      for(int i = 1; i <= f; i++)
        r *= i;

      return r;
    }
  }

  private class PlayerTable extends JPanel {
    ExtendedTableLayout tl;

    JLabel[][] playerRow;

    JLabel[] countStart;
    JLabel[] countNoStart;

    boolean hPM = false;

    public PlayerTable() {
      super();

      //setBorder(new javax.swing.border.LineBorder(Color.blue,1));

      setLayout(tl = new ExtendedTableLayout(new double[][]
        {{TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED},
         {TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED}}));

      tl.setVGap(7);
      tl.setHGap(7);

      String[] pl = new String[]
        {"MJ","BR","KAJ","LB","PM","DW","OR"};

      playerRow = new JLabel[21][];

      JLabel lbl;

      int r = 0;
      for(int i = 0; i < pl.length; i++)
        for(int j = i+1; j < pl.length; j++)
          for(int k = j+1; k < pl.length; k++)
            for(int l = k+1; l < pl.length; l++)
              for(int m = l+1; m < pl.length; m++) {
                playerRow[r] = new JLabel[5];
                add(playerRow[r][0] = new JLabel(pl[i]),(i+1)+","+r);
                add(playerRow[r][1] = new JLabel(pl[j]),(j+1)+","+r);
                add(playerRow[r][2] = new JLabel(pl[k]),(k+1)+","+r);
                add(playerRow[r][3] = new JLabel(pl[l]),(l+1)+","+r);
                add(playerRow[r][4] = new JLabel(pl[m]),(m+1)+","+r);
                add((lbl = new JLabel(++r + "")), "0," + (r-1));
                lbl.setForeground(new Color(0,0,0,.5f));
              }

      countStart = new JLabel[r];
      countNoStart = new JLabel[r];


      tl.insertColumn(8,10);
      tl.insertColumn(9,20);
      //tl.insertColumn(8,TableLayout.PREFERRED);
      tl.insertColumn(10,10);
      tl.insertColumn(11,20);
      //tl.insertColumn(10,TableLayout.PREFERRED);

      for(int i = 0; i < countStart.length; i++) {
        add(countNoStart[i] = new JLabel(""),"9,"+i);        
        add(countStart[i] = new JLabel(""),"11,"+i);
      }
    }

    public void paintComponent(Graphics g) {
      super.paintComponent(g);

      if(hPM) {
        Graphics2D g2 = (Graphics2D) g.create();
        
        g2.setColor(Color.white);
        
        float x = 0;
        
        for(int i = 0; i < 5; i++) {
          x += tl.getColumnPixels(i) + tl.getHGap();
        }
        
        g2.fill(new java.awt.geom.Rectangle2D.Float
                (x,0f,
                 tl.getColumnPixels(5),(float)getHeight()));
        
        g2.dispose();
      }
    }
    

    /*
    public void addNotify() {
      super.addNotify();
      for(int i = 0; i < countStart.length; i++) {
        countNoStart[i].setPreferredSize(countNoStart[i].getSize());
        countStart[i].setPreferredSize(countStart[i].getSize());
        countNoStart[i].setContent("");
        countStart[i].setContent("");
      }
      
    }
    */

    public void colorRow(int i, Color c) {
      for(int j = 0; j < 5; j++)
        playerRow[i][j].setForeground(c);
      countStart[i].setForeground(c);
      countNoStart[i].setForeground(c);
    }
  }

  private class PascalAnimation extends Animation {
    public PascalAnimation() {
      super(cp, 500);

      setPartArray(new AnimationPart[]
        {
          new Part0(),
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5(),
          new Part6(),
        });
    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.append("We start with the left-hand side of the identity and list all 21 ways to choose 5 starting players.",NEWLINES);

        total.setVisible(true);
        playerTable.setVisible(true);
      }
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        details.append("Now, we count the right-hand side by counting the number of starting line-ups in two different groups: where PM does not start and where PM does start.",
                       NEWLINES|SET_MARK|SCROLL);

        playerTable.hPM = true;
        playerTable.repaint();        
      }
    }    

    private class Part2 implements AnimationPart {
      public void animate() {
        details.append("First, count the number of starting line-ups where PM does not start.",NEWLINES|SET_MARK|SCROLL);

        int rows[] = {1,2,5,8,12,17};

        for(int i = 0; i < rows.length; i++) {
          playerTable.colorRow(rows[i],Color.RED);
          playerTable.countNoStart[rows[i]].setText((i+1)+"");
          sleep();
        }

        noStart.setVisible(true);
      }
    }

    private class Part3 implements AnimationPart {
      public void animate() {
        details.append("Next, count the number of starting line-ups where PM does start.",NEWLINES|SET_MARK|SCROLL);

        int rows[] = {0,3,4,6,7,9,10,11,13,14,15,16,18,19,20};

        for(int i = 0; i < rows.length; i++) {
          playerTable.colorRow(rows[i],Color.BLUE);
          playerTable.countStart[rows[i]].setText((i+1)+"");
          sleep();
        }

        start.setVisible(true);
      }
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        details.append("Counting either way gives us the same total of 21. It is clear in this example that", NEWLINES|SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("7","5"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("21"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getBinomialCoefficient("7\u22121","5"),
              MathSymbol.getTextSymbol("+"),
              MathSymbol.getBinomialCoefficient("7\u22121","5-1")},20f),
           SCROLL);
        sum.setVisible(true);
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        details.setContent("We can also visualize this using Pascal's triangle. Recall that the entry at the 7th row and 5th column will be");
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("7","5"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("21")},20f));

        left.removeAll();
        left.add(pascalTriangle,"0,0,C,C");
        left.validate();
        left.repaint();
        PascalVisualization.this.validate();
        PascalVisualization.this.repaint();        

        pascalTriangle.colorEntry(7,5,Color.GREEN.darker().darker());
      }
    }

    private class Part6 implements AnimationPart {
      public void animate() {
        details.append("Now notice the 4th and 5th entries in the 6th row. The triangle shows that these sum to 21.",SET_MARK|SCROLL);

        sleep();
        pascalTriangle.colorEntry(6,4,Color.BLUE);
        sleep();
        pascalTriangle.colorEntry(6,5,Color.RED);

      }      
    }    

  }
}
