package modules.tulip;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Line2D;

import java.io.*;

import vcp.mathnotation.*;
import vcp.utilities.ModuleUtilities;
import vcp.utilities.ExtendedTableLayout;
import vcp.utilities.ScrollableTextArea;
import vcp.animation.*;

public class TulipVisualization extends JPanel 
  implements 
    vcp.utilities.ScrollableTextAreaConstants {

  ControlPanel cp;

  TulipTable tulipTable;

  ScrollableTextArea details;

  public TulipVisualization() {
    super(new ExtendedTableLayout
          (new double [][]
            {{ExtendedTableLayout.FILL},
             {ExtendedTableLayout.PREFERRED,
              ExtendedTableLayout.PREFERRED,
              ExtendedTableLayout.FILL,
              ExtendedTableLayout.PREFERRED}}));

    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));
    details.setMargins(6);   

    MathExpression expr = new MathExpression
      (new MathSymbol[]
        {MathSymbol.getSummationSymbol
         ("b=0","k",MathSymbol.getBinomialCoefficient
          ("n + (k\u2212b)","(k\u2212b)").toMathExpression(),
          "sum of C(n+(k-b),(k-b)) with b = 0 to k"),
         MathSymbol.getTextSymbol("="),
         MathSymbol.getBinomialCoefficient
         ("n+k+1","k")},30f);

    JLabel theorem = new JLabel
      (new ImageIcon(expr.toImage(10)));

    add(theorem,"0,0,c,c");
    add(tulipTable = new TulipTable(), "0,1,c,c");
    add(details.getJScrollPane(),"0,2");
    add(cp = new SliderControlPanel(), "0,3");

    new TulipAnimation();
  }

  private class TulipTable extends JPanel {
    Tulip[][] tulips;
    JLabel[] blabel;
    MathExpression[] bexpr;

    ExtendedTableLayout tl;

    int line = 0;

    public TulipTable() {
      super();

      double[] c = new double[5];
      double[] r = new double[10];
      int i;

      for(i = 0; i < c.length; i++)
        c[i] = ExtendedTableLayout.PREFERRED;
      for(i = 0; i < r.length; i++)
        r[i] = ExtendedTableLayout.PREFERRED;

      setLayout(tl = new ExtendedTableLayout(c,r));

      tl.setVGap(10);
      tl.setHGap(10);


      tulips = new Tulip[10][];
      for(i = 0; i < 10; i++)
        tulips[i] = new Tulip[]
          {new Tulip(Tulip.BLUE),
           new Tulip(Tulip.BLUE),
           new Tulip(Tulip.BLUE),
           new Tulip(Tulip.BLUE),
           new Tulip(Tulip.BLUE)};

      
      i = 0;
      for(int j = 0; j < 4; j++)
        for(int k = j+1; k < 5; k++) {
          tulips[i][j] = new Tulip(Tulip.RED);
          tulips[i++][k] = new Tulip(Tulip.RED);
        }

      for(i = 0; i < 10; i++)
        for(int j = 0; j < 5; j++)
          add(tulips[i][j],j+","+i);

      blabel = new JLabel[4];
      bexpr = new MathExpression[4];
      
      /*
      bexpr[0] = new MathExpression
        (new MathSymbol[]
          {MathSymbol.getBinomialCoefficient
           ("1 + (3\u22120)","(3\u22120)"),
           MathSymbol.getTextSymbol("="),
           MathSymbol.getTextSymbol("4")},30f);
      blabel[0] = new JLabel
        (new ImageIcon
         (bexpr[0].toImage(getBackground(),10)));
      bexpr[1] = new MathExpression
        (new MathSymbol[]
          {MathSymbol.getBinomialCoefficient
           ("1 + (3\u22121)","(3\u22121)"),
           MathSymbol.getTextSymbol("="),
           MathSymbol.getTextSymbol("3")},30f);
      blabel[1] = new JLabel
        (new ImageIcon
         (bexpr[1].toImage(getBackground(),10)));
      bexpr[2] = new MathExpression
        (new MathSymbol[]
          {MathSymbol.getBinomialCoefficient
           ("1 + (3\u22122)","(3\u22122)"),
           MathSymbol.getTextSymbol("="),
           MathSymbol.getTextSymbol("2")},30f);
      blabel[2] = new JLabel
        (new ImageIcon
         (bexpr[2].toImage(getBackground(),10)));
      bexpr[3] = new MathExpression
        (new MathSymbol[]
          {MathSymbol.getBinomialCoefficient
           ("1 + (3\u22123)","(3\u22123)"),
           MathSymbol.getTextSymbol("="),
           MathSymbol.getTextSymbol("1")},30f);
      blabel[3] = new JLabel
        (new ImageIcon
         (bexpr[3].toImage(getBackground(),10)));
      */

      for(i = 0; i < blabel.length; i++) {
        blabel[i] = new JLabel("b = " + i);
        blabel[i].setVisible(false);
      }

      tl.insertColumn(0,ExtendedTableLayout.PREFERRED);
      add(blabel[0],"0,0,0,3");      
      add(blabel[1],"0,4,0,6");      
      add(blabel[2],"0,7,0,8");      
      add(blabel[3],"0,9,0,9");

      tl.insertRow(10,10);
    }

    public void paintComponent(Graphics g) {
      super.paintComponent(g);

      Graphics2D g2d = (Graphics2D) g.create();

      g2d.setColor(Color.BLACK);
      g2d.setStroke(new BasicStroke(2));
      
      int[] lines = new int[] {4,7,9};
      
      double y =  -(double)(tl.getVGap())/2.0;
      int i = 0;
      for(int j = 0; j < line; j++) {
        y += tl.getRowPixels(i++) + tl.getVGap();
        for(; i < lines[j]; i++)
          y += tl.getRowPixels(i) + tl.getVGap();
        g2d.draw
          (new Line2D.Double(0,y,getWidth(),y));
      }      
    }

    public void setSeparators(int i) {
      line = i;
      repaint();
    }

    public void highlightTulipRow(int i, boolean v) {
      for(int j = 0; j < tulips[i].length; j++) {
        tulips[i][j].highlight(v);
        tulips[i][j].repaint();
      }
    }

    public void highlightBlabel(int i, boolean v) {
      if(v) {
        //bexpr[i].setColor(Color.blue);
        blabel[i].setForeground(Color.blue);
        //blabel[i].setIcon
        //  (new ImageIcon(bexpr[i].toImage(getBackground(),10)));
        blabel[i].setVisible(true);
      } else {
        //bexpr[i].setColor(Color.black);
        blabel[i].setForeground(Color.black);
        //blabel[i].setIcon
        //  (new ImageIcon(bexpr[i].toImage(getBackground(),10)));
      }
    }
  }

  private class Tulip extends JLabel {
    static final int RED = 0;
    static final int BLUE = 1;

    final ImageIcon RED_NORMAL = 
      new ImageIcon(ModuleUtilities.getModuleResource
                    ("tulip/tulip_red_dark.png"));

    final ImageIcon BLUE_NORMAL  = 
      new ImageIcon(ModuleUtilities.getModuleResource
                    ("tulip/tulip_blue_dark.png"));

    final ImageIcon RED_HIGHLIGHT = 
      new ImageIcon(ModuleUtilities.getModuleResource
                    ("tulip/tulip_red.png"));

    final ImageIcon BLUE_HIGHLIGHT  = 
      new ImageIcon(ModuleUtilities.getModuleResource
                    ("tulip/tulip_blue.png"));

    int color;

    public Tulip(int c) {
      super();

      setIcon(c == RED ? RED_NORMAL : BLUE_NORMAL);

      color = c;
    }

    public void highlight(boolean v) {
      if(v)
        setIcon(color == RED ? RED_HIGHLIGHT : BLUE_HIGHLIGHT);
      else
        setIcon(color == RED ? RED_NORMAL : BLUE_NORMAL);
        
    }
  }

  private class TulipAnimation extends Animation {
    public TulipAnimation() {
      super(cp);

      setPartArray(new AnimationPart[]
        {
          new Part0(),
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5(),
          new Part6(),
        });
    }

    private class Part0 implements AnimationPart {
      public void animate() {
        details.setContent("Suppose Barb has 2 red tulip bulbs (n = 1) and 3 blue tulip bulbs (k = 3). She is interested in the number of visually distinct ways she can plant these in one row. We start with all these ways which is the right-hand side of the identity,");
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("1+3+1","3"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("10")},20f));
      }
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        details.append("Now, we will look at the left-hand side and count this number based on how many blue tulips are first.",SET_MARK|NEWLINES|SCROLL);     
      }
    }    

    private class Part2 implements AnimationPart {
      public void animate() {
        details.append("The number of ways to plant the tulips so there are 0 blue tulips first is",SET_MARK|NEWLINES);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient
              ("1 + (3\u22120)","(3\u22120)"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("4")},20f),SCROLL);

        for(int i = 0; i < 4; i++)
          tulipTable.highlightTulipRow(i,true);
        tulipTable.highlightBlabel(0,true);

        tulipTable.setSeparators(1);
      }
    }

    private class Part3 implements AnimationPart {
      public void animate() {
        details.append("The number of ways to plant the tulips so there is 1 blue tulip first is",SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient
              ("1 + (3\u22121)","(3\u22121)"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("3")},20f),SCROLL);

        for(int i = 0; i < 4; i++)
          tulipTable.highlightTulipRow(i,false);
        tulipTable.highlightBlabel(0,false);

        for(int i = 4; i < 7; i++)
          tulipTable.highlightTulipRow(i,true);
        tulipTable.highlightBlabel(1,true);

        tulipTable.setSeparators(2);
      }
    }

    private class Part4 implements AnimationPart {
      public void animate() {
        details.append("The number of ways to plant the tulips so there are 2 blue tulips first is",SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient
              ("1 + (3\u22122)","(3\u22122)"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("2")},20f),SCROLL);

        for(int i = 4; i < 7; i++)
          tulipTable.highlightTulipRow(i,false);
        tulipTable.highlightBlabel(1,false);

        for(int i = 7; i < 9; i++)
          tulipTable.highlightTulipRow(i,true);
        tulipTable.highlightBlabel(2,true);

        tulipTable.setSeparators(3);
      }
    }

    private class Part5 implements AnimationPart {
      public void animate() {
        details.append("The number of ways to plant the tulips so there are 3 blue tulips first is",SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient
              ("1 + (3\u22123)","(3\u22123)"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("1")},20f),SCROLL);

        for(int i = 7; i < 9; i++)
          tulipTable.highlightTulipRow(i,false);
        tulipTable.highlightBlabel(2,false);

        tulipTable.highlightTulipRow(9,true);
        tulipTable.highlightBlabel(3,true);
      }
    }

    private class Part6 implements AnimationPart {
      public void animate() {
        tulipTable.highlightTulipRow(9,false);
        tulipTable.highlightBlabel(3,false);     

        details.append("It is clear in this example that,",SET_MARK);
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getSummationSymbol
              ("b=0","3",MathSymbol.getBinomialCoefficient
               ("1 + (3\u2212b)","(3\u2212b)").toMathExpression(),
               "sum of C(1+(3-b),(3-b)) with b = 0 to 3"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("10"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getBinomialCoefficient
              ("1+3+1","3")},20f),SCROLL);
      }
    }
  }
}
