package modules.vandermonde;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.util.Hashtable;

import vcp.mathnotation.*;
import vcp.animation.*;
import vcp.utilities.ScrollableTextArea;

import info.clearthought.layout.TableLayout;

public class VandermondeVisualization extends JPanel 
  implements MouseListener,vcp.utilities.ScrollableTextAreaConstants {

  static final boolean TESTING = true;

  Hashtable ballComboHash;

  BallPair[] ballCombo;

  Ball first;

  int count = 0;

  ControlPanel cp;

  BallPanel bp;

  JLabel[] blueBallCount;
  JLabel total;

  ScrollableTextArea details;

  public VandermondeVisualization() {
    setLayout(new TableLayout
              (new double[][]
                {{TableLayout.FILL,
                  TableLayout.PREFERRED,
                  TableLayout.PREFERRED,
                  TableLayout.FILL},
                 {TableLayout.PREFERRED,
                  //TableLayout.PREFERRED,
                  TableLayout.FILL,
                  TableLayout.PREFERRED}}));

    
    ballCombo = new BallPair[15];
    ballComboHash = new Hashtable(30);

    int index = 0;
    for(int i = 1; i <= 4; i++)
      for(int j = i+1; j <= 4; j++) {
        ballCombo[index] = new BallPair("o"+i+"o"+j);
        ballComboHash.put("o"+i+"o"+j,new Integer(index));
        ballComboHash.put("o"+j+"o"+i,new Integer(index++));
      }

    for(int i = 1; i <= 2; i++)
      for(int j = 1; j <= 4; j++) {
        ballCombo[index] = new BallPair("b"+i+"o"+j);
        ballComboHash.put("b"+i+"o"+j,new Integer(index));
        ballComboHash.put("o"+j+"b"+i,new Integer(index++));
      }

    ballCombo[index] = new BallPair("b1b2");
    ballComboHash.put("b1b2",new Integer(index));
    ballComboHash.put("b2b1",new Integer(index));

    for(int i = 0; i < ballCombo.length; i++) {
      ballCombo[i].setShown(false);   
      ballCombo[i].setVisible(false);
    }

    JPanel comboPanel = new JPanel();
    
    double[] r = new double[15];

    for(int i = 0; i < r.length; i++)
      r[i] = TableLayout.PREFERRED;

    TableLayout tl;
    comboPanel.setLayout
      (tl = new TableLayout
       (new double[] 
         {TableLayout.PREFERRED,
          TableLayout.PREFERRED,
          TableLayout.PREFERRED},
        r));
    
    tl.setVGap(2);

    for(int i = 0; i < ballCombo.length; i++) {
      comboPanel.add(ballCombo[i],"1,"+i);
    }

    blueBallCount = new JLabel[3];

    blueBallCount[0] = new JLabel
      (new ImageIcon
       (new MathExpression
        (new MathSymbol[]
          {MathSymbol.getBinomialCoefficient("2","0"),
           MathSpace.getNoSpace(),
           MathSymbol.getBinomialCoefficient("4","2\u22120"),
           MathSymbol.getTextSymbol("="),
           MathSymbol.getTextSymbol("6")
          },20f).toImage(10)));

    blueBallCount[1] = new JLabel
      (new ImageIcon
       (new MathExpression
        (new MathSymbol[]
          {MathSymbol.getBinomialCoefficient("2","1"),
           MathSpace.getNoSpace(),
           MathSymbol.getBinomialCoefficient("4","2\u22121"),
           MathSymbol.getTextSymbol("="),
           MathSymbol.getTextSymbol("8")
          },20f).toImage(10)));
    
    blueBallCount[2] = new JLabel
      (new ImageIcon
       (new MathExpression
        (new MathSymbol[]
          {MathSymbol.getBinomialCoefficient("2","2"),
           MathSpace.getNoSpace(),
           MathSymbol.getBinomialCoefficient("4","2\u22122"),
           MathSymbol.getTextSymbol("="),
           MathSymbol.getTextSymbol("1")
          },20f).toImage(10)));

    blueBallCount[0].setVisible(false);
    blueBallCount[1].setVisible(false);
    blueBallCount[2].setVisible(false);

    total = new JLabel
      (new ImageIcon
       (new MathExpression
        (new MathSymbol[]
          {MathSymbol.getBinomialCoefficient("2+4","2"),
           MathSymbol.getTextSymbol("="),
           MathSymbol.getTextSymbol("15")
          },25f).toImage(10)));
    total.setVisible(false);
    
    comboPanel.add(blueBallCount[0],"0,0,0,5,c,c");
    comboPanel.add(blueBallCount[1],"0,6,0,13,c,c");
    comboPanel.add(blueBallCount[2],"0,14,c,t");
    comboPanel.add(total,"2,0,2,14,c,c");

    
    
    bp = new BallPanel();

    cp = new SliderControlPanel();

    new VandermondeAnimation();

    details = new ScrollableTextArea();
    details.getJScrollPane().setPreferredSize(new java.awt.Dimension(20,100));
    details.setMargins(6);

    JLabel identity = new JLabel
      (new ImageIcon
       (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("m+n","r"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSummationSymbol
              ("i = 0", "r", new MathExpression
               (new MathSymbol[]
                 {MathSymbol.getBinomialCoefficient("m","i"),
                  MathSpace.getNoSpace(),
                  MathSymbol.getBinomialCoefficient("n","r\u2212i")
                 }),
               "Sum of C(m,i)*C(n,r-i) with i = 0 to r")
             },25f).toImage(10)));

    add(bp,"1,0,c,c");
    add(identity,"1,0,l,t");
    add(comboPanel,"2,0,2,0,r,c");
    add(details.getJScrollPane(),"0,1,3,1");
    add(cp,"0,2,3,2");

    first = null;
  }

  private class BallPair extends JPanel {
    Color color1;
    Color color2;

    String labelText1;
    String labelText2;

    Font font1;
    Font font2;

    float labelWidth1;
    float labelWidth2;
    
    boolean highlight;

    boolean shown;

    public BallPair(String pair) {
      String[] list;
      list = new String[5];
      list[1]=pair.substring(0,1);
      list[2]=pair.substring(1,2);
      list[3]=pair.substring(2,3);
      list[4]=pair.substring(3,4);
      
      color1 = getColor(list[1]);
      labelText1 = list[2];
      color2 = getColor(list[3]);
      labelText2 = list[4];

      Dimension d = new Dimension(50,20);
      setPreferredSize(d);
      setSize(d);
      setMinimumSize(d);
      setMaximumSize(d);
    }

    public void paintComponent(Graphics g) {
      super.paintComponent(g);

      final BasicStroke stroke = new BasicStroke(2);

      Graphics2D g2 = (Graphics2D) g.create();
      
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                          RenderingHints.VALUE_ANTIALIAS_ON);
      g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                          RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

      GlyphVector label1;
      GlyphVector label2;

      if(font1 == null || font2 == null) {
        font1 = new Font("SansSerif",Font.BOLD,12);
        font2 = new Font("SansSerif",Font.BOLD,12);

        label1 = font1.createGlyphVector
          (g2.getFontRenderContext(),labelText1);        
        label2 = font2.createGlyphVector
          (g2.getFontRenderContext(),labelText2);        

        float fh = (float)label1.getOutline(0f,0f).getBounds2D().getHeight();
        font1 = font1.deriveFont((12f*8f)/fh);

        fh = (float)label2.getOutline(0f,0f).getBounds2D().getHeight();
        font2 = font2.deriveFont((12f*8f)/fh);

        label1 = font1.createGlyphVector
          (g2.getFontRenderContext(),labelText1);
        label2 = font2.createGlyphVector
          (g2.getFontRenderContext(),labelText2);

        labelWidth1 = (float)label1.getOutline(0f,0f).getBounds2D().getWidth();
        labelWidth2 = (float)label2.getOutline(0f,0f).getBounds2D().getWidth();
      }
      
      label1 = font1.createGlyphVector
        (g2.getFontRenderContext(),labelText1);
      label2 = font2.createGlyphVector
        (g2.getFontRenderContext(),labelText2);
      
      Arc2D.Float arc1 = new Arc2D.Float
        (2f,2f,(float)(20-4),(float)(20-4),0,360,Arc2D.OPEN);
      Arc2D.Float arc2 = new Arc2D.Float
        (30f,2f,(float)(20-4),(float)(20-4),0,360,Arc2D.OPEN);

      //g2.setColor(Color.white);
      g2.setStroke(stroke);
      //g2.fill(arc1);
      //g2.fill(arc2);

      if(shown) {
        g2.setColor(color1);
        g2.fill(arc1);
        
        if(highlight) {
          g2.setColor(new Color(~color1.getRGB()));
        }
        
        g2.draw(arc1);   
        
        g2.setColor(new Color(~color1.getRGB()));
        g2.drawGlyphVector(label1, 10 - labelWidth1/2f, 14);      
        
        g2.setColor(color2);
        g2.fill(arc2);
        if(highlight) {
          g2.setColor(new Color(~color2.getRGB()));
        }
        g2.draw(arc2);
        
        g2.setColor(new Color(~color2.getRGB()));
        g2.drawGlyphVector(label2, 38 - labelWidth2/2f, 14);      
      }

      else {
        g2.setColor(new Color(0f,0f,0f,.3f));
        g2.draw(arc1);
        g2.draw(arc2);
      }
        
        
        g2.dispose();
    }

    private Color getColor(String c) {
      if(c.equals("o"))
        return Color.orange;
      else if(c.equals("b"))
        return Color.blue;
      else
        //System.out.println("null " + c);
        return null;
      
    }

    public void setShown(boolean s) {
      shown = s;
      repaint();
    }

    public boolean isShown() {
      return shown;
    }
  }

  private class BallPanel extends JPanel {
    Ball[] balls;

    public BallPanel() {
      final double EXTRA = 30;
      final double STANDARD = 35;

      setLayout(new TableLayout
                (new double[][]
                  {{EXTRA,
                    STANDARD,STANDARD,STANDARD,STANDARD,
                    EXTRA},
                   {EXTRA,
                    STANDARD,STANDARD,STANDARD,STANDARD,
                    EXTRA}}));


      Dimension d = new Dimension(200,200);
      setPreferredSize(d);
      setSize(d);
      setMinimumSize(d);
      setMaximumSize(d);

      balls = new Ball[6];

      add(balls[0] = new Ball("1","o"),"1,3,1,4,c,c");
      add(balls[1] = new Ball("2","o"),"1,1,2,2,c,c");
      add(balls[2] = new Ball("3","o"),"3,3,c,c");
      add(balls[3] = new Ball("4","o"),"4,2,c,c");
      add(balls[4] = new Ball("1","b"),"2,4,c,c");
      add(balls[5] = new Ball("2","b"),"3,1,c,c");

      for(int i = 0; i < 6; i++)
        balls[i].setEnabled(false);
    }

    public void paintComponent(Graphics g) {
      super.paintComponent(g);

      final BasicStroke stroke = new BasicStroke(2);

      Graphics2D g2 = (Graphics2D) g.create();

      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                          RenderingHints.VALUE_ANTIALIAS_ON);

      Arc2D.Float arc = new Arc2D.Float
        (2f,2f,(float)(200-4),(float)(200-4),0,360,Arc2D.OPEN);

      g2.setColor(Color.white);
      g2.setStroke(stroke);
      g2.fill(arc);

      g2.setColor(Color.black);
      g2.draw(arc);

      g2.dispose();
    }    
  }

  private class Ball extends JPanel {
    String labelText;

    Color color;

    boolean highlight;
    boolean down;

    Font font;
    float labelWidth;
    
    String name;

    public Ball(String l, String c) {
      name = c+l;
      
      labelText = l;

      if (c.equals("o"))
        color = Color.orange;
      else if (c.equals("b"))
        color = Color.blue;

      setBackground(Color.white);

      font = null;

      Dimension d = new Dimension(30,30);
      setPreferredSize(d);
      setSize(d);
      setMinimumSize(d);
      setMaximumSize(d);

      addMouseListener(VandermondeVisualization.this);
    }

    public void paintComponent(Graphics g) {
      super.paintComponent(g);
      final BasicStroke stroke = new BasicStroke(2);

      Graphics2D g2 = (Graphics2D) g.create();

      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                          RenderingHints.VALUE_ANTIALIAS_ON);
      g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                          RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

      GlyphVector label;

      if(font == null) {
        font = new Font("SansSerif",Font.BOLD,12);
        //FontMetrics fm = g.getFontMetrics(font);
        label = font.createGlyphVector
          (g2.getFontRenderContext(),labelText);        
        float fh = (float)label.getOutline(0f,0f).getBounds2D().getHeight();
        font = font.deriveFont((12f*10f)/fh);
        label = font.createGlyphVector
          (g2.getFontRenderContext(),labelText);        
        labelWidth = (float)label.getOutline(0f,0f).getBounds2D().getWidth();
      }
      
      label = font.createGlyphVector
        (g2.getFontRenderContext(),labelText);

      g2.setStroke(stroke);

      Arc2D.Float arc = new Arc2D.Float
        (2f,2f,(float)(getWidth()-4),(float)(getHeight()-4),0,360,Arc2D.OPEN);

      g2.setColor(color);
      g2.draw(arc);
      if(highlight) {
        g2.fill(arc);
        g2.setColor(new Color(~color.getRGB()));
      }


      g2.drawGlyphVector(label, 15 - labelWidth/2f, 20);

      g2.dispose();
    }
  }

  private class VandermondeAnimation extends Animation {
    public VandermondeAnimation() {
      super(cp, 1000, 400, 500);

      setPartArray(new AnimationPart[]
        {
          new Part1(),
          new Part2(),
          new Part3(),
          new Part4(),
          new Part5(),
        });
    }

    private class Part1 implements AnimationPart {
      public void animate() {
        details.setContent("Suppose there are 2 blue balls (m) and 4 yellow balls (n) in a bag. You want to pull 2 out (r). How many ways can this be done? Start by counting the left-hand side of the identity. To do this, choose pairs of balls from the bag by clicking on each ball in the pair. As you choose pairs, they will appear to the right. Keep choosing pairs until you have all possible pairs.");

        for(int i = 0; i < bp.balls.length; i++)
          bp.balls[i].setEnabled(true);

        for(int i = 0; i < ballCombo.length; i++)
          ballCombo[i].setVisible(true);

        synchronized(bp) {
          try {
            bp.wait();
          } catch(InterruptedException e) {}
        }

        total.setVisible(true);

        details.append("There are 15 possible pairs. \n\nNow, we will count this number using the right-hand side of the identity. The number of ways to choose a pair of balls is counted based on how many blue balls are in the pair.",NEWLINES|SET_MARK|SCROLL);
      }
    }

    private class Part2 implements AnimationPart {
      public void animate() {
        int i = 0;
        for(; i < 6; i++) {
          ballCombo[i].highlight = true;
          ballCombo[i].repaint();
        }
        blueBallCount[0].setVisible(true);
        
        details.append("There are 6 pairs with 0 blue balls (i = 0).",SET_MARK|NEWLINES|SCROLL);
      }
    }

    private class Part3 implements AnimationPart {
      public void animate() {
        int i = 6;
        for(int j = 0; j < i; j++) {
          ballCombo[j].highlight = false;
          ballCombo[j].repaint();
        }
        for(; i < 14; i++) {
          ballCombo[i].highlight = true;
          ballCombo[i].repaint();
        }
        blueBallCount[1].setVisible(true);

        details.append("There are 8 pairs with 1 blue ball (i = 1).",SET_MARK|NEWLINES|SCROLL);
      }
    }    

    private class Part4 implements AnimationPart {
      public void animate() {
        for(int j = 6; j < 14; j++) {
          ballCombo[j].highlight = false;
          ballCombo[j].repaint();
        }
        ballCombo[14].highlight = true;
        ballCombo[14].repaint();
        blueBallCount[2].setVisible(true);

        details.append("There is 1 pair with 2 blue balls (i = 2).",SET_MARK|NEWLINES|SCROLL);
      }
    }    

    private class Part5 implements AnimationPart {
      public void animate() {
        ballCombo[14].highlight = false;
        ballCombo[14].repaint(); 

        details.append("It is clear that in this example,",NEWLINES|SET_MARK);
        
        details.append
          (new MathExpression
           (new MathSymbol[]
             {MathSymbol.getBinomialCoefficient("2+4","2"),
              MathSymbol.getTextSymbol("="),
              MathSymbol.getTextSymbol("15"),  
              MathSymbol.getTextSymbol("="),
              MathSymbol.getSummationSymbol
              ("i = 0", "2", new MathExpression
               (new MathSymbol[]
                 {MathSymbol.getBinomialCoefficient("2","i"),
                  MathSpace.getNoSpace(),
                  MathSymbol.getBinomialCoefficient("4","2\u2212i")
                 }),
               "Sum of C(2,i)*C(4,2-i) with i = 0 to 2")
             },25f),SCROLL);
      }
    }
  }


  public void mouseClicked(MouseEvent e) {}
  
  public void mousePressed(MouseEvent e) {
    Ball b = (Ball) e.getSource();
    if(!b.isEnabled())
      return;

    if(!b.highlight) {
      b.highlight = true;
      b.down = true;
      b.repaint();
    }
  }
  
  public void mouseReleased(MouseEvent e) {
    final Ball b = (Ball) e.getSource();
    if(!b.isEnabled())
      return;

    //********  needs to go away
    /*
    if(TESTING) {
      for(int i = 0; i < bp.balls.length; i++) {
        bp.balls[i].setEnabled(false);
        bp.balls[i].highlight = false;
        bp.balls[i].repaint();
      }
      for(int i = 0; i < ballCombo.length; i++)
        ballCombo[i].setShown(true);

      synchronized(bp) {
        bp.notifyAll();
      }
      return;
    }
    */
    //*********  end needs to go away

    if(b.down) {
      if(first == null) {
        first = b;
      } else {

        final Ball f = first;
        (new Thread() {
            public void run() {
              try {
                sleep(200);
              } catch(InterruptedException e) {}

              f.highlight = false;
              b.highlight = false;
              f.repaint();
              b.repaint();
            }
          }).start();

        int index = 
          ((Integer)ballComboHash.get(first.name + b.name)).intValue();

        if(!ballCombo[index].isShown()) {
          ballCombo[index].setShown(true);
          if(++count >= 15) {
            for(int i = 0; i < bp.balls.length; i++)
              bp.balls[i].setEnabled(false);
            synchronized(bp) {
              bp.notifyAll();
            }
          }
        }
        first = null;
      }
    }

    b.down = false;
  }
  
  public void mouseEntered(MouseEvent e) {}
  
  public void mouseExited(MouseEvent e) {
    Ball b = (Ball) e.getSource();
    if(!b.isEnabled())
      return;

    if(b.down) {
      b.highlight = false;
      b.repaint();
    }
    
    b.down = false;
  }
}
