package vcp.platform;

import java.io.File;
import java.io.BufferedInputStream;
import java.io.FileInputStream;

/**
 * A class loader to load module visualization classes. This loader
 * will load from the working directory of the application.
 *
 * @author phil kaasa
 * @version 1.0
 */
public class ModuleClassLoader extends ClassLoader {
  String location;

  /**
   * Creates a new <code>ModuleClassLoader</code> instance.
   *
   * Uses the current path to search for classes
   */
  public ModuleClassLoader() {
    this("");
  }

  /**
   * Creates a new <code>ModuleClassLoader</code> instance.
   *
   * @param l the path to look for classes in
   */
  public ModuleClassLoader(String l) {
    super();

    location = l;
  }
  
  public Class findClass(String name) 
    throws ClassNotFoundException {
    
    byte[] b = loadClassData(name);
    return defineClass(name,b,0,b.length);
    
  }
  
  /**
   * load class data from the class file
   *
   * @param name the class to load
   * @return an array of bytes containing the class data
   * @exception ClassNotFoundException if the class cannot be found
   */
  private byte[] loadClassData(String name)
    throws ClassNotFoundException {
    
    try {
      File f = new File(location + name.replaceAll("[.]","/") + ".class");
      
      byte[] b = new byte[(int)f.length()];
      
      (new BufferedInputStream(new FileInputStream(f))).read(b);
      
      return b;
    } catch(Exception e) {
      throw new ClassNotFoundException
        (name + ".class file could not be loaded", e);
    }
  }
}