package vcp.platform;

import javax.swing.JRadioButton;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.ButtonGroup;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.io.FileOutputStream;

import java.util.Properties;

import vcp.utilities.ExceptionDialog;

import info.clearthought.layout.TableLayout;

public class PDFPrefs extends JPanel implements ActionListener {
  private JFrame frame;

  private JRadioButton dfault;
  private JRadioButton custom;
  private JTextField path;
  private JButton browse;
  private JButton ok;
  private JButton cancel;
  private JFileChooser fileChooser;

  public PDFPrefs() {
    super(new TableLayout(new double[][]
      {{TableLayout.FILL,
        TableLayout.PREFERRED,
        TableLayout.PREFERRED},
       {TableLayout.PREFERRED,
        TableLayout.PREFERRED,
        TableLayout.PREFERRED,
        TableLayout.PREFERRED,
        TableLayout.PREFERRED}}));

    ButtonGroup bg = new ButtonGroup();

    dfault = new JRadioButton("Default Viewer");
    custom = new JRadioButton("Specify a Viewer");
    bg.add(dfault);
    bg.add(custom);
    dfault.addActionListener(this);
    custom.addActionListener(this);
    
    path = new JTextField(25);
    path.addActionListener(this);

    browse = new JButton("Browse...");
    browse.addActionListener(this);

    if(GUIActions.getProperty("pdf.viewer").equalsIgnoreCase("default")) {
      dfault.setSelected(true);
      path.setEnabled(false);
      browse.setEnabled(false);
    } else {
      custom.setSelected(true);
      path.setText(GUIActions.getProperty("pdf.viewer"));
    }

    ok = new JButton("OK");
    cancel = new JButton("Cancel");
    ok.addActionListener(this);
    cancel.addActionListener(this);

    add(dfault,"0,0,2,0");
    add(custom,"0,1,2,0");
    add(new JLabel("Path to PDF Viewer Application:"),"0,2,2,2");
    add(path,"0,3,1,3");
    add(browse, "2,3");
    add(ok,"1,4");
    add(cancel,"2,4");

    fileChooser = new JFileChooser();

    frame = new JFrame("VCP PDF Preferences");
    frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    frame.getContentPane().add(this);
    frame.pack();
  }

  public void setVisible(boolean b) {
    super.setVisible(b);

    frame.setVisible(b);
  }

  public void actionPerformed(ActionEvent ev) {
    Object source = ev.getSource();

    if(source == dfault || source == custom) {
      path.setEnabled(custom.isSelected());
      browse.setEnabled(custom.isSelected());
    }

    else if(source == browse) {
      int retVal = fileChooser.showOpenDialog(frame);
      
      if(retVal == JFileChooser.APPROVE_OPTION)
        path.setText(fileChooser.getSelectedFile().getPath());
    }

    else if(source == ok || source == path) {
      if(dfault.isSelected())
        GUIActions.setProperty("pdf.viewer","default");
      else
        GUIActions.setProperty("pdf.viewer",path.getText());

      try {
        Properties justPDF = new Properties();

        justPDF.setProperty("pdf.viewer",
                            GUIActions.getProperty("pdf.viewer"));

        justPDF.store
          (new FileOutputStream("vcp.cfg"),"");
      } catch (Exception e) {
        ExceptionDialog.showExceptionDialog
          ("Configuration Store Error",
           "There was an error saving the configuration.\n\nConfiguration changes will not be saved.",e);
      }

      frame.dispose();
    }
    
    else if(source == cancel) {
      frame.dispose();
    }
  }

  public static void main(String[] args) {
    (new PDFPrefs()).setVisible(true);
  }
}