package vcp.utilities;

import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;

import java.net.URL;
import java.net.MalformedURLException;

import vcp.platform.GUIActions;

/**
 * Some utilities that a module might need to use.
 *
 * @author phil kaasa
 * @version 1.0
 */
public class ModuleUtilities {
  private ModuleUtilities() {

  }

  /**
   * <p>Returns an applet-safe URL to the resource. The path is
   * relative to the root of where the modules are stored. However,
   * you cannot descend below that root
   * (e.g. <code>../other_directory</code> is invalid). For example,
   * to access a resource in a module's whose package is
   * <code>example</code>, one might call this method like so,</p>
   *
   * <p><code>ModuleUtilities.getModuleResource(getClass(),
   * "example/image.jpg")</p>
   *
   * @param s the path to resource
   * @return a <code>URL</code> to the passed resource
   * @see #getModuleResourceAsStream(String)
   */
  public static URL getModuleResource(String s) {
    URL url;

    if(GUIActions.vcp.isApplet()) {
      url =  ModuleUtilities.class.getResource("/modules/" + s);
    } else {
      try {
        url =  new URL(GUIActions.getRoot(),"modules/" + s);
      } catch (MalformedURLException e) {
        url =  null;
      }
    }

    //System.out.println(url);
    return url;
  }

  /**
   * <p>Returns an applet-safe <code>InputStream</code> for the
   * resource. The path is relative to the root of where the modules
   * are stored. However, you cannot descend below that root
   * (e.g. <code>../other_directory</code> is invalid).
   *
   * @param s a <code>String</code> value
   * @return an <code>InputStream</code> value
   * @see #getModuleResource(String)
   */
  public static InputStream getModuleResourceAsStream(String s) {
    InputStream is;

    if(GUIActions.vcp.isApplet()) {
      is = ModuleUtilities.class.getResourceAsStream("/modules/" + s);
    } else {
      try {
        is = new FileInputStream(GUIActions.getRootAsString() + "modules/" + s);
      } catch(IOException e) {
        is = null;
      }
    }

    return is;
  }
}