package vcp.utilities;

import java.awt.Font;

/**
 * Thrown in situations where no suitable font can found
 *
 * @author phil kaasa
 * @version 1.0
 */
public class NoFontFoundException extends Exception {
  /** the undisplayable characters for each available font */
  protected char[][] badChars;

  /** the available fonts */
  protected Font[] fonts;

  /**
   * Creates a new <code>NoFontFoundException</code> instance.
   *
   * @param msg the exception message
   * @param f an array of <code>Font</code>s tried.
   * @param bc a two dimensional array of undisplayable characters for
   * each of the <code>Font</code>s tried.
   */
  public NoFontFoundException(String msg, Font[] f, char[][] bc) {
    super(msg);
    
    badChars = bc;
    fonts = f;
  }

  /**
   * Returns an array of fonts tried. This array only includes fonts
   * that are actually on the system.
   *
   * @return the list of fonts tried
   */
  public Font[] getFontList() {
    return fonts;
  }

  /**
   * Returns an array of undisplayable characters. This array
   * corresponds to the font name at index i int the font list.
   *
   * @param i index of the font whose undisplayable characters is desired
   * @return an array of undisplayable characters
   */
  public char[] getBadChars(int i) {
    return badChars[i];
  }

  /**
   * Returns the index of the font name which can display the largest
   * number of characters from the test string.
   *
   * @return a font name index
   */
  public int getBestFont() {
    int min = 0;
    int minL = badChars[0].length;

    for(int i = 1; i < badChars.length; i++) {
      if(badChars[i].length < minL) {
        minL = badChars[i].length;
        min = i;
      }
    }

    return min;
  }
}