package vcp.utilities;

import javax.swing.JLabel;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Font;
import java.awt.RenderingHints;
import java.awt.font.GlyphVector;
import java.awt.font.LineMetrics;

/**
 * A label with text rotated 90 degrees. Not all JLabel methods are
 * fully implemented.
 *
 * @author phil kaasa
 * @version 1.0
 */
public class VerticalJLabel extends JLabel {
  public VerticalJLabel(String s) {
    super(s);
  }

  public void paintComponent(Graphics gfx) {
    Graphics2D g2d = (Graphics2D) gfx;

    g2d.setColor(getForeground());
    g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                         RenderingHints.VALUE_TEXT_ANTIALIAS_DEFAULT);

    GlyphVector g = getFont().createGlyphVector
      (g2d.getFontRenderContext(), getText());

    Rectangle2D b = g2d.getFontMetrics().getStringBounds(getText(),g2d);

    double x = (getWidth()+b.getHeight())/2.0 - 
      g2d.getFontMetrics().getLineMetrics(getText(),gfx).getDescent();
    double y = getHeight();

    g2d.rotate(-.5*Math.PI,x,y);
    g2d.drawGlyphVector(g,(float)x,(float)y);
    g2d.rotate(.5*Math.PI,x,y);
  }

  public Dimension getPreferredSize() {
    Rectangle2D r = getGraphics().getFontMetrics().
      getStringBounds(getText(),getGraphics());

   return new Dimension((int)r.getHeight(),(int)r.getWidth());
  }

  /**
   * for testing
   *
   * @param args a <code>String[]</code> value
   */
  public static void main(String[] args) {
    javax.swing.JFrame f = new javax.swing.JFrame();
    javax.swing.JPanel p = new javax.swing.JPanel
      (new java.awt.GridLayout(1,2));

    p.add(new VerticalJLabel("T y h j g a K M"));
    //p.add(new JLabel("Testing"));

    f.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
    f.getContentPane().add(p);
    f.pack();
    f.setVisible(true);
  }
}